''' <summary>
''' This class was created to provide the possibility of showing data without using any additional grid controls.
''' </summary>
Public Class CQGMiniGrid

    Private m_Cols As Integer
    Private m_Rows As Integer
    Private m_Panel As Panel
    Private m_Labels()() As Label

    Private Const TOP As Integer = 20
    Private Const LEFT As Integer = 0
    Private Const HEIGHT As Integer = 18
    Private Const WIDTH As Integer = 60

    ''' <summary>
    ''' Create labels and add them to a given groupbox.
    ''' </summary>
    ''' <param name="cols"> 
    ''' Number of labels (vertical).
    ''' </param>
    ''' <param name="rows">
    ''' Number of labels (horizontal).
    ''' </param>
    ''' <param name="parent">
    ''' GroupBox, in which labels will be added.
    ''' </param>
    Public Sub New(ByVal cols As Integer, ByVal rows As Integer, ByRef parent As GroupBox)
        If cols < 1 Then
            Throw New ArgumentException(cols & " is invalid argument.")
        End If
        If rows < 1 Then
            Throw New ArgumentException(rows & " is invalid argument.")
        End If

        m_Cols = cols
        m_Rows = rows

        m_Panel = New Panel
        m_Panel.Height = parent.Size.Height - 48
        m_Panel.Location = New Point(7, 38)
        m_Panel.BorderStyle = BorderStyle.None
        m_Panel.AutoScroll = True
        parent.Controls.Add(m_Panel)

        ReDim m_Labels(m_Cols)
        For j As Integer = 0 To m_Cols - 1
            ReDim m_Labels(j)(m_Rows)
            For i As Integer = 0 To m_Rows
                m_Labels(j)(i) = New Label
                m_Labels(j)(i).Size = New Size(WIDTH, HEIGHT)
                m_Labels(j)(i).UseMnemonic = False
                m_Labels(j)(i).BorderStyle = BorderStyle.Fixed3D
                If i = 0 Then
                    parent.Controls.Add(m_Labels(j)(i))
                Else
                    m_Panel.Controls.Add(m_Labels(j)(i))
                End If
            Next
        Next
        ReDraw()
    End Sub

    ''' <summary>
    ''' Recalculates sizes of labels and relocates all lables.
    ''' </summary>
    Public Sub ReDraw()
        ReLocate()
      ' if autoscrolls are being shown, leaves the horizontal one only.
        If m_Labels(0)(m_Rows).Location.Y + HEIGHT > m_Panel.Height Then
            For i As Integer = 0 To m_Rows
                m_Labels(m_Cols - 1)(i).Width -= 16
            Next
            m_Panel.Width = m_Labels(m_Cols - 1)(m_Rows).Location.X + m_Labels(m_Cols - 1)(m_Rows).Size.Width + 16
        Else
            m_Panel.Width = m_Labels(m_Cols - 1)(m_Rows).Location.X + m_Labels(m_Cols - 1)(m_Rows).Size.Width
        End If

    End Sub

    ''' <summary>
    ''' Calculates locations of labels and sets their coordinates. 
    ''' </summary>
    Private Sub ReLocate()
        'Header part
        m_Labels(0)(0).Location = New Point(m_Panel.Location.X, TOP)
        For j As Integer = 1 To m_Cols - 1
            m_Labels(j)(0).Location = New Point(m_Labels(j - 1)(0).Location.X + m_Labels(j - 1)(0).Size.Width, TOP)
        Next
        'Fields in Panel
        For i As Integer = 1 To m_Rows
            m_Labels(0)(i).Location = New Point(LEFT, HEIGHT * (i - 1))
            For j As Integer = 1 To m_Cols - 1
                m_Labels(j)(i).Location = New Point(m_Labels(j - 1)(i).Location.X + _
                                                    m_Labels(j - 1)(i).Size.Width, _
                                                    m_Labels(j - 1)(i).Location.Y)
            Next
        Next
    End Sub

    ''' <summary>
    ''' Sets the specified column's HeaderValue.
    ''' </summary>
    ''' <param name="col">
    ''' The specified column.
    ''' </param>
    Public WriteOnly Property HeaderValue(ByVal col As Integer) As String
        Set(ByVal Value As String)
            If col < 0 Or col >= m_Cols Then
                Throw New IndexOutOfRangeException(col & " is invalid column index")
            End If
            m_Labels(col)(0).Text = Value
            m_Labels(col)(0).TextAlign = ContentAlignment.MiddleCenter
            m_Labels(col)(0).Font = New Font("Arial", 9, FontStyle.Bold)
        End Set
    End Property

    ''' <summary>
    ''' Sets the specified column's alignment.
    ''' </summary>
    ''' <param name="col">
    ''' The specified column.
    ''' </param>
    Public WriteOnly Property ColumnAlign(ByVal col As Integer) As ContentAlignment
        Set(ByVal Value As ContentAlignment)
            If col < 0 Or col >= m_Cols Then
                Throw New IndexOutOfRangeException(col & " is invalid column index")
            End If
            For i As Integer = 1 To m_Rows
                m_Labels(col)(i).TextAlign = Value
            Next
        End Set
    End Property

    ''' <summary>
    ''' Set specified column's width.
    ''' </summary>
    ''' <param name="col">
    ''' The specified column.
    ''' </param>
    Public WriteOnly Property ColumnWidth(ByVal col As Integer) As Integer
        Set(ByVal Value As Integer)
            If col < 0 Or col >= m_Cols Then
                Throw New IndexOutOfRangeException(col & " is invalid column index")
            End If
            For i As Integer = 0 To m_Rows
                m_Labels(col)(i).Width = Value
            Next
        End Set
    End Property

    ''' <summary>
    ''' Clear the content of labels in the specified column.
    ''' </summary>
    ''' <param name="col">
    ''' Column, content of which must be cleared.
    ''' </param>
    Public Sub ClearColumn(ByVal col As Integer)
        If col < 0 Or col >= m_Cols Then
            Throw New IndexOutOfRangeException(col & " is invalid column index")
        End If
        For i As Integer = 1 To m_Rows
            m_Labels(col)(i).Text = ""
        Next
    End Sub

    ''' <summary>
    ''' Clears the content of all labels.
    ''' </summary>
    Public Sub Clear()
        For i As Integer = 0 To m_Cols - 1
            ClearColumn(i)
        Next
    End Sub

    ''' <summary>
    ''' Gets the number of rows in grid.
    ''' </summary>
    ''' <returns>
    ''' The number of rows in grid.
    ''' </returns>
    Public ReadOnly Property Rows() As Integer
        Get
            Return m_Rows
        End Get
    End Property

    ''' <summary>
    ''' Gets/sets the item for the specified row and column.
    ''' </summary>
    ''' <param name="row">
    ''' The specified row.
    ''' </param>
    ''' <param name="col">
    ''' The specified column.
    ''' </param>
    ''' <returns>
    ''' The item specified by the row and column.
    ''' </returns>
    Public Property Item(ByVal row As Integer, ByVal col As Integer) As Label
        Get
            If col < 0 Or col >= m_Cols Then
                Throw New IndexOutOfRangeException(col & " is invalid column index")
            End If
            If row < 0 Or row >= m_Rows Then
                Throw New IndexOutOfRangeException(row & " is invalid row index")
            End If
            Return m_Labels(col)(row + 1)
        End Get
        Set(ByVal Value As Label)
            If col < 0 Or col >= m_Cols Then
                Throw New IndexOutOfRangeException(col & " is invalid column index")
            End If
            If row < 0 Or row >= m_Rows Then
                Throw New IndexOutOfRangeException(row & " is invalid row index")
            End If
            m_Labels(col)(row + 1) = Value
        End Set
    End Property

End Class
