﻿using CQG;
using System;
using System.Windows;

namespace CreateOrderByInstrumentName
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Fields
        /// <summary>
        /// CQG API instance (CQGCEL).
        /// </summary>
        private CQGCEL _cel;

        /// <summary>
        /// Used to keep track on last placed order.
        /// </summary>
        private string _orderGUID;

        /// <summary>
        /// Represents internal window state.
        /// </summary>
        private EState _state = EState.Starting;

        /// <summary>
        /// Status text to wait GW connection.
        /// </summary>
        private const string WAITING_GW_STATUS_TEXT = "Waiting for connection to GW...";
        #endregion

        #region Constructors
        public MainWindow()
        {
            InitializeComponent();
            initializeCEL();
        }
        #endregion

        #region Enums
        internal enum EState
        {
            /// <summary>
            /// Initializing MainWindow
            /// </summary>
            Starting,

            /// <summary>
            /// Window awaits user input
            /// </summary>
            Idle,

            /// <summary>
            /// Waiting for some CEL event.
            /// </summary>
            WorkInProgress
        }
        #endregion

        #region UI methods
        /// <summary>
        /// Sets new value to _state and updates status message if one is given.
        /// </summary>
        /// <param name="state">New state to set.</param>
        /// <param name="statusMessage">New status message, optional. Default is null.</param>
        private void setState(EState state, string statusMessage = null)
        {
            _state = state;
            if (statusMessage != null)
            {
                statusTextBlock.Text = string.Format("Status: {0}", statusMessage);
            }
            updateUI();
        }

        /// <summary>
        /// Updates IsEnabled property for controls depending on internal state.
        /// </summary>
        private void updateUI()
        {
            if (_state == EState.Starting)
            {
                // Controls may not be initialized at this point.
                return;
            }

            var isEnabled = _state == EState.Idle;
            symbolEntry.IsEnabled = isEnabled;

            // allow place order if have parameters
            isEnabled = isEnabled && symbolEntry.Text.Length > 0;
            buyButton.IsEnabled = isEnabled;
            sellButton.IsEnabled = isEnabled;
        }

        /// <summary>
        /// Shows error message box with proveded text.
        /// </summary>
        /// <param name="errorMessage">Error text to show.</param>
        private void showError(string errorMessage)
        {
            _ = MessageBox.Show(errorMessage, "Place MKT Order - Error", MessageBoxButton.OK, MessageBoxImage.Error);
        }

        /// <summary>
        /// Shuts down CEL.
        /// </summary>
        private void onWindowClosing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            try
            {
                _cel.Shutdown();
            }
            catch (Exception ex)
            {
                showError(ex.Message);
            }
        }

        /// <summary>
        /// Invokes UI update.
        /// </summary>
        private void onSymbolEntryTextChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            updateUI();
        }

        /// <summary>
        /// But button click event handler.
        /// </summary>
        private void onBuyButtonClick(object sender, RoutedEventArgs e)
        {
            placeOrder(symbolEntry.Text, 1);
        }

        /// <summary>
        /// Sell button click event handler.
        /// </summary>
        private void onSellButtonClick(object sender, RoutedEventArgs e)
        {
            placeOrder(symbolEntry.Text, -1);
        }
        #endregion

        #region CEL-related methods
        /// <summary>
        /// Initializes CQGCEL object.
        /// </summary>
        private void initializeCEL()
        {
            setState(EState.WorkInProgress);
            _cel = new CQGCEL();

            try
            {
                _cel.DataConnectionStatusChanged +=
                    new _ICQGCELEvents_DataConnectionStatusChangedEventHandler(onDataConnectionStatusChanged);
                _cel.GWConnectionStatusChanged +=
                    new _ICQGCELEvents_GWConnectionStatusChangedEventHandler(onGWConnectionStatusChanged);
                _cel.AccountChanged += new _ICQGCELEvents_AccountChangedEventHandler(onAccountChanged);
                _cel.OrderChanged += new _ICQGCELEvents_OrderChangedEventHandler(onOrderChanged);
                _cel.APIConfiguration.ReadyStatusCheck = eReadyStatusCheck.rscOff;
                _cel.Startup();
            }
            catch (Exception e)
            {
                showError(e.Message + "\nFailed to initialize CEL. Application will be closed.");
                Application.Current.Shutdown();
            }
        }

        /// <summary>
        /// Simplified data connection event handler.
        /// </summary>
        /// <param name="newStatus">New connection status.</param>
        private void onDataConnectionStatusChanged(eConnectionStatus newStatus)
        {
            setState(EState.WorkInProgress, eConnectionStatus.csConnectionUp == newStatus
                ? WAITING_GW_STATUS_TEXT : "Waiting for connection to IC...");
        }

        /// <summary>
        /// Simplified GW connection event handler. Sets new Account subscription level.
        /// </summary>
        /// <param name="newStatus">New connection status.</param>
        private void onGWConnectionStatusChanged(eConnectionStatus newStatus)
        {
            if (newStatus == eConnectionStatus.csConnectionUp)
            {
                try
                {
                    _cel.AccountSubscriptionLevel = eAccountSubscriptionLevel.aslAccountUpdatesAndOrders;
                }
                catch (Exception e)
                {
                    showError(e.Message + "\nFailed to request accounts. Application will be closed.");
                    Application.Current.Shutdown();
                    return;
                }

                setState(EState.WorkInProgress, "Waiting for accounts loading...");
            }
            else
            {
                setState(EState.WorkInProgress, WAITING_GW_STATUS_TEXT);
            }
        }

        /// <summary>
        /// Simplified account or position information change event handler.
        /// </summary>
        /// <param name="change">Account change type which allows to differentiate the occurred changes.</param>
        /// <param name="account">A CQGAccount object representing the account to which the current change refers.</param>
        /// <param name="position">A CQGPosition object representing the position to which the current change refers.</param>
        private void onAccountChanged(eAccountChangeType change, CQGAccount account, CQGPosition position)
        {
            if (change != eAccountChangeType.actAccountsReloaded)
            {
                return;
            }

            setState(EState.Idle, "Ready to place order.");
        }

        /// <summary>
        /// Simplified order change event handler.
        /// </summary>
        /// <param name="change">The type of the occurred change.</param>
        /// <param name="order">CQGOrder object representing the order to which the change refers.</param>
        /// <param name="oldProperties">
        /// CQGOrderProperties collection representing the old values of the changed order properties.
        /// </param>
        /// <param name="fill">CQGFill object representing the last fill of the order.</param>
        /// <param name="error">
        /// CQGError object representing either an error sent by CQG Gateway or an intermediate error.
        /// </param>
        private void onOrderChanged(eChangeType change, CQGOrder order, CQGOrderProperties oldProperties,
                                      CQGFill fill, CQGError error)
        {
            try
            {
                if (_cel.IsValid(error))
                {
                    setState(EState.Idle, "Last order has failed.");
                    showError(string.Format("Error with order occurred: code={0}, description: {1}",
                                            error.Code, error.Description));
                    return;
                }

                if (_orderGUID != order.GUID)
                {
                    // ignore this order, we are watching for another one
                    return;
                }

                var gwStatus = (eOrderStatus)order.Properties[eOrderProperty.opGWStatus].Value;
                var newState =
                    change == eChangeType.ctChanged && gwStatus == eOrderStatus.osInOrderBook ? EState.Idle : _state;
                var statusText = string.Format("Order GW status: {0}, quantity: {1}, instrument: {2}",
                                               gwStatus, order.Properties[eOrderProperty.opQuantityFractional].Value,
                                               order.Properties[eOrderProperty.opInstrumentName].Value);
                setState(newState, statusText);
            }
            catch (Exception e)
            {
                showError("onOrderChanged caught exception: " + e.Message);
                setState(EState.Idle, "Exception has been caught while processing last order update.");
                return;
            }
        }

        /// <summary>
        /// Creates and places MKT order with provided symbol and quantity.
        /// </summary>
        /// <param name="symbol">Full symbol with prefix and month-year suffix to place order on.</param>
        private void placeOrder(string symbol, int quantity)
        {
            try
            {
                // simplified logic: take first account
                var account = _cel.Accounts.ItemByIndex[0];
                var order = _cel.CreateOrderByInstrumentName(eOrderType.otMarket, symbol, account, quantity);
                _orderGUID = order.GUID;
                order.Place();
            }
            catch (Exception e)
            {
                showError(e.Message);
                return;
            }

            setState(EState.WorkInProgress, "Order has been sent...");
        }
        #endregion
    }
}
