using System.Diagnostics;
using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;

/// <summary>
/// This class was created to provide the possibility of showing data without using any additional grid controls.
/// </summary>
namespace InstrumentProperties
{
   public class CQGMiniGrid
   {
      private int m_Cols;
      private int m_Rows;
      private Label[][] m_Labels;

      /// <summary>
      /// Create labels and add them to given groupbox.
      /// </summary>
      /// <param name="cols">
      /// Number of labels (vertical).
      /// </param>
      /// <param name="rows">
      /// Number of labels (horizontal).
      /// </param>
      /// <param name="parent">
      /// GroupBox, in which labels will be added.
      /// </param>
      public CQGMiniGrid(int cols, int rows, GroupBox parent)
      {
         if (cols < 1)
         {
            throw new ArgumentException(cols.ToString() + " is invalid argument.");
         }
         if (rows < 1)
         {
            throw new ArgumentException(rows.ToString() + " is invalid argument.");
         }

         m_Cols = cols;
         m_Rows = rows;

         m_Labels = new Label[m_Cols + 1][];
         for (int j = 0; j < m_Cols; j++)
         {
            m_Labels[j] = new Label[m_Rows + 1];
            for (int i = 0; i <= m_Rows; i++)
            {
               m_Labels[j][i] = new Label();
               m_Labels[j][i].Size = new Size(60, 18);
               m_Labels[j][i].UseMnemonic = false;
               m_Labels[j][i].BorderStyle = BorderStyle.Fixed3D;
               parent.Controls.Add(m_Labels[j][i]);
            }
         }
         ReLocate();
      }

      /// <summary>
      /// Calculate locations of labels and set their coordinates.
      /// Also, change font and alignment for the header row.
      /// </summary>
      private void ReLocate()
      {
         for (int i = 0; i <= m_Rows; i++)
         {
            m_Labels[0][i].Location = new Point(8, 20 + m_Labels[0][i].Size.Height * i);
            m_Labels[0][0].TextAlign = ContentAlignment.MiddleCenter;
            m_Labels[0][0].Font = new Font("Arial", 9, FontStyle.Bold);
            for (int j = 1; j < m_Cols; j++)
            {
               if (i == 0)
               {
                  m_Labels[j][i].Location = new Point(m_Labels[j - 1][i].Location.X +
                                             m_Labels[j - 1][i].Size.Width, 20);
               }
               else
               {
                  m_Labels[j][i].Location = new Point(m_Labels[j - 1][i].Location.X +
                                             m_Labels[j - 1][i].Size.Width,
                                             m_Labels[j][i - 1].Location.Y +
                                             m_Labels[j][i - 1].Size.Height);
               }
            }
         }
      }

      /// <summary>
      /// Set the specified column's HeaderValue.
      /// </summary>
      /// <param name="col">
      /// The specified column.
      /// </param>
      /// <param name="val">
      /// New HeaderValue of the specified column.
      /// </param>
      public void SetHeaderValue(int col, string val)
      {
         if (col < 0 || col >= m_Cols)
         {
            throw (new IndexOutOfRangeException(col.ToString() + " is invalid column index"));
         }
         m_Labels[col][0].Text = val;
         m_Labels[col][0].TextAlign = ContentAlignment.MiddleCenter;
         m_Labels[col][0].Font = new Font("Arial", 9, FontStyle.Bold);
      }

      /// <summary>
      /// Set the specified column's alignment.
      /// </summary>
      /// <param name="col">
      /// The specified column.
      /// </param>
      /// <param name="val">
      /// New alignment of the specified column.
      /// </param>
      public void SetColumnAlign(int col, ContentAlignment val)
      {
         if (col < 0 || col >= m_Cols)
         {
            throw (new IndexOutOfRangeException(col.ToString() + " is invalid column index"));
         }
         for (int i = 1; i <= m_Rows; i++)
         {
            m_Labels[col][i].TextAlign = val;
         }
      }

      /// <summary>
      /// Set specified column's width.
      /// </summary>
      /// <param name="col">
      /// The specified column.
      /// </param>
      /// <param name="val">
      /// New width of the specified column.
      /// </param>
      public void SetColumnWidth(int col, int val)
      {
         if (col < 0 || col >= m_Cols)
         {
            throw (new IndexOutOfRangeException(col.ToString() + " is invalid column index"));
         }
         for (int i = 0; i <= m_Rows; i++)
         {
            m_Labels[col][i].Width = val;
         }
         ReLocate();
      }

      /// <summary>
      /// Clear the content of labels in the specified column.
      /// </summary>
      /// <param name="col">
      /// Column, content of which must be cleared.
      /// </param>
      public void ClearColumn(int col)
      {
         if (col < 0 || col >= m_Cols)
         {
            throw (new IndexOutOfRangeException(col.ToString() + " is invalid column index"));
         }
         for (int i = 1; i <= m_Rows; i++)
         {
            m_Labels[col][i].Text = "";
         }
      }

      /// <summary>
      /// Clear the content of all labels.
      /// </summary>
      public void Clear()
      {
         for (int i = 0; i < m_Cols; i++)
         {
            ClearColumn(i);
         }
      }

      /// <summary>
      /// Get the number of rows in grid.
      /// </summary>
      /// <returns>
      /// The number of rows in grid.
      /// </returns>
      public int Rows
      {
         get
         {
            return m_Rows;
         }
      }

      /// <summary>
      /// Get/set the item for the specifed row and column.
      /// </summary>
      /// <param name="row">
      /// The specifed row.
      /// </param>
      /// <param name="col">
      /// The specifed column.
      /// </param>
      public Label this[int row, int col]
      {
         get
         {
            if (col < 0 || col >= m_Cols)
            {
               throw (new IndexOutOfRangeException(col.ToString() + " is invalid column index"));
            }
            if (row < 0 || row >= m_Rows)
            {
               throw (new IndexOutOfRangeException(row.ToString() + " is invalid row index"));
            }
            return m_Labels[col][row + 1];
         }
         set
         {
            if (col < 0 || col >= m_Cols)
            {
               throw (new IndexOutOfRangeException(col.ToString() + " is invalid column index"));
            }
            if (row < 0 || row >= m_Rows)
            {
               throw (new IndexOutOfRangeException(row.ToString() + " is invalid row index"));
            }
            m_Labels[col][row + 1] = value;
         }
      }

   }

}
