''' <summary>
''' This class was created to provide the possibility of showing data without using any additional grid controls.
''' </summary>
Public Class CQGMiniGrid

   Private m_Cols As Integer
   Private m_Rows As Integer
   Private m_Labels()() As Label

   ''' <summary>
   ''' Create labels and add them to given groupbox.
   ''' </summary>
   ''' <param name="cols"> 
   ''' Number of labels (vertical).
   ''' </param>
   ''' <param name="rows">
   ''' Number of labels (horizontal).
   ''' </param>
   ''' <param name="parent">
   ''' GroupBox, in which labels will be added.
   ''' </param>
   Public Sub New(ByVal cols As Integer, ByVal rows As Integer, ByRef parent As GroupBox)
      If cols < 1 Then
         Throw New ArgumentException(cols.ToString() & " is invalid argument.")
      End If
      If rows < 1 Then
         Throw New ArgumentException(rows.ToString() & " is invalid argument.")
      End If

      m_Cols = cols
      m_Rows = rows

   ReDim m_Labels(m_Cols)
   For j As Integer = 0 To m_Cols - 1
      ReDim m_Labels(j)(m_Rows)
      For i As Integer = 0 To m_Rows
         m_Labels(j)(i) = New Label
         m_Labels(j)(i).Size = New Size(60, 18)
         m_Labels(j)(i).UseMnemonic = False
         m_Labels(j)(i).BorderStyle = BorderStyle.Fixed3D
         parent.Controls.Add(m_Labels(j)(i))
      Next
   Next
   ReLocate()
   End Sub

   ''' <summary>
   ''' Calculate locations of labels and set their coordinates. 
   ''' Also, change font and alignment for the header row.
   ''' </summary>
   Private Sub ReLocate()
      For i As Integer = 0 To m_Rows
         m_Labels(0)(i).Location = New Point(8, 20 + m_Labels(0)(i).Size.Height * i)
         m_Labels(0)(0).TextAlign = ContentAlignment.MiddleCenter
         m_Labels(0)(0).Font = New Font("Arial", 9, FontStyle.Bold)
         For j As Integer = 1 To m_Cols - 1
            If i = 0 Then
               m_Labels(j)(i).Location = New Point(m_Labels(j - 1)(i).Location.X + _
                                         m_Labels(j - 1)(i).Size.Width, 20)
            Else
               m_Labels(j)(i).Location = New Point(m_Labels(j - 1)(i).Location.X + _
                                         m_Labels(j - 1)(i).Size.Width, _
                                         m_Labels(j)(i - 1).Location.Y + _
                                       m_Labels(j)(i - 1).Size.Height)
            End If
         Next
      Next
   End Sub

   ''' <summary>
   ''' Set the specified column's HeaderValue.
   ''' </summary>
   ''' <param name="col">
   ''' The specified column.
   ''' </param>
   Public WriteOnly Property HeaderValue(ByVal col As Integer) As String
      Set(ByVal Value As String)
         If col < 0 Or col >= m_Cols Then
          Throw New IndexOutOfRangeException(col.ToString() & " is invalid column index")
         End If
         m_Labels(col)(0).Text = Value
         m_Labels(col)(0).TextAlign = ContentAlignment.MiddleCenter
         m_Labels(col)(0).Font = New Font("Arial", 9, FontStyle.Bold)
      End Set
   End Property

   ''' <summary>
   ''' Set the specified column's alignment.
   ''' </summary>
   ''' <param name="col">
   ''' The specified column.
   ''' </param>
   Public WriteOnly Property ColumnAlign(ByVal col As Integer) As ContentAlignment
      Set(ByVal Value As ContentAlignment)
         If col < 0 Or col >= m_Cols Then
            Throw New IndexOutOfRangeException(col.ToString() & " is invalid column index")
         End If
         For i As Integer = 1 To m_Rows
            m_Labels(col)(i).TextAlign = Value
         Next
      End Set
   End Property

   ''' <summary>
   ''' Set specified column's width.
   ''' </summary>
   ''' <param name="col">
   ''' The specified column.
   ''' </param>
   Public WriteOnly Property ColumnWidth(ByVal col As Integer) As Integer
      Set(ByVal Value As Integer)
         If col < 0 Or col >= m_Cols Then
            Throw New IndexOutOfRangeException(col.ToString() & " is invalid column index")
         End If
         For i As Integer = 0 To m_Rows
            m_Labels(col)(i).Width = Value
         Next
         ReLocate()
      End Set
   End Property

   ''' <summary>
   ''' Clear the content of labels in the specified column.
   ''' </summary>
   ''' <param name="col">
   ''' Column, content of which must be cleared.
   ''' </param>
   Public Sub ClearColumn(ByVal col As Integer)
      If col < 0 Or col >= m_Cols Then
         Throw New IndexOutOfRangeException(col.ToString() & " is invalid column index")
      End If
      For i As Integer = 1 To m_Rows
         m_Labels(col)(i).Text = ""
      Next
   End Sub

   ''' <summary>
   ''' Clear the content of all labels.
   ''' </summary>
   Public Sub Clear()
      For i As Integer = 0 To m_Cols - 1
         ClearColumn(i)
      Next
   End Sub

   ''' <summary>
   ''' Get the number of rows in grid.
   ''' </summary>
   ''' <returns>
   ''' The number of rows in grid.
   ''' </returns>
   Public ReadOnly Property Rows() As Integer
      Get
         Return m_Rows
      End Get
   End Property

   ''' <summary>
   ''' Get/set the item for the specifed row and column.
   ''' </summary>
   ''' <param name="row">
   ''' The specifed row.
   ''' </param>
   ''' <param name="col">
   ''' The specifed column.
   ''' </param>
   ''' <returns>
   ''' The item specifed by the row and column.
   ''' </returns>
   Public Property Item(ByVal row As Integer, ByVal col As Integer) As Label
      Get
         If col < 0 Or col >= m_Cols Then
            Throw New IndexOutOfRangeException(col.ToString() & " is invalid column index")
         End If
         If row < 0 Or row >= m_Rows Then
            Throw New IndexOutOfRangeException(row.ToString() & " is invalid row index")
         End If
         Return m_Labels(col)(row + 1)
      End Get
      Set(ByVal Value As Label)
         If col < 0 Or col >= m_Cols Then
            Throw New IndexOutOfRangeException(col.ToString() & " is invalid column index")
         End If
         If row < 0 Or row >= m_Rows Then
            Throw New IndexOutOfRangeException(row.ToString() & " is invalid row index")
         End If
         m_Labels(col)(row + 1) = Value
      End Set
   End Property

End Class
