// EventHandlersCQGCEl.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "InstrumentProperties.h"
#include "InstrumentPropertiesDlg.h"
#include "QFormulasDlg.h"

namespace
{
   /// \brief Shows message box with symbol resolution failed reason.
   /// \param symbol - Requested symbol
   /// \param cqg_error - Error occured during a symbol resolution process
   void onSymbolResolutonFailed(BSTR symbol, ICQGError* cqg_error)
   {
      ATL::CComBSTR errDescription;
      auto hr = cqg_error->get_Description(&errDescription);
      AssertCOMError<ICQGError>(cqg_error, hr);

      CStringW msg;
      msg.AppendFormat(L"%s resolution failed. %s", symbol, errDescription.m_str);
      AfxMessageBox(ATL::CW2T(msg), MB_ICONWARNING);
   }
}

STDMETHODIMP CInstrumentPropertiesDlg::OnDataError(
                                             LPDISPATCH /*obj*/, 
                                             BSTR errorDescription)
{
   CString errMsg(_T("Data error: "));
   errMsg += ATL::CW2T(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CInstrumentPropertiesDlg::OnCELStarted()
{
   // Disable controls until data connection goes up
   EnableControls(FALSE);

   return S_OK;
}

STDMETHODIMP CInstrumentPropertiesDlg::OnDataConnectionStatusChanged(
                                             eConnectionStatus newStatus)
{
   try
   {
      m_lbDataConnection.SetWindowText(_T("Data connection : ") + ConnectionStatusToString(newStatus));

      EnableControls(TRUE);   
   } 
   catch (const std::runtime_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()));
      return E_FAIL;
   }
   return S_OK;
}

STDMETHODIMP CInstrumentPropertiesDlg::OnInstrumentResolved(
                                          BSTR symbol,
                                          ICQGInstrument* instrument,
                                          ICQGError* cqg_error)
{
   try
   {
      VARIANT_BOOL errorOccured = VARIANT_FALSE;
      auto hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &errorOccured);
      AssertCOMError<ICQGError>(cqg_error, hr);

      if (errorOccured)
      {
         onSymbolResolutonFailed(symbol, cqg_error);
         return E_FAIL;
      }

      ATL::CComBSTR fullName;
      hr = instrument->get_FullName(&fullName);
      AssertCOMError<ICQGInstrument>(instrument, hr);

      // If instrument is already subscribed do not add it to list
      if (m_lsInstruments.FindStringExact(0, ATL::CW2T(fullName)) != LB_ERR)
         return S_FALSE;

      // Add name to list
      m_lsInstruments.AddString(ATL::CW2T(fullName));
      
      m_btRemoveAll.EnableWindow(TRUE);
   }
   catch (const std::runtime_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()));
      return E_FAIL;
   }
   return S_OK;
}

STDMETHODIMP CInstrumentPropertiesDlg::OnInstrumentChanged(
                                             ICQGInstrument* instrument, 
                                             ICQGQuotes* quotes, 
                                             ICQGInstrumentProperties* props)
{
   try
   {
      ATL::CComBSTR fullname;
      HRESULT hr = instrument->get_FullName(&fullname);
      AssertCOMError<ICQGInstrument>(instrument, hr);

      if (fullname != GetSelectedFullName())
      {
         // Not selected instrument was changed
         return S_OK;
      }

      UpdateQuotes(instrument, quotes);
      UpdateProperties(instrument, props);
   }
   catch (const std::runtime_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()));
      return E_FAIL;
   }
   return S_OK;
}

STDMETHODIMP CInstrumentPropertiesDlg::OnInstrumentDOMChanged(
                                             ICQGInstrument* instrument, 
                                             ICQGDOMQuotes* /*prevAsks*/, 
                                             ICQGDOMQuotes* /*prevBids*/)
{
   try
   {
      ATL::CComBSTR fullname;
      HRESULT hr = instrument->get_FullName(&fullname);
      AssertCOMError<ICQGInstrument>(instrument, hr);

      if (fullname != GetSelectedFullName())
      {
         // Not selected instrument was changed
         return S_OK;
      }

      UpdateDOM(instrument);
   }
   catch (const std::runtime_error& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()));
      return E_FAIL;
   }
   return S_OK;
}

STDMETHODIMP CQFormulasDlg::OnQFormulaDefinitionsResolved(
                              ICQGQFormulaDefinitions* cqg_qformula_definitions, 
                              ICQGError* cqg_error)
{
   try
   {
      // Raise error, when any error occurred during request
      VARIANT_BOOL isValid;
      HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &isValid);
      AssertCOMError(m_spCQGCEL, hr);

      if (isValid == VARIANT_TRUE)
      {
         ATL::CComBSTR errDescription;
         hr = cqg_error->get_Description(&errDescription);
         AssertCOMError<ICQGError>(cqg_error, hr);

         AfxMessageBox(ATL::CW2T(errDescription), MB_ICONEXCLAMATION);

         return S_OK;
      }

      m_spQFormulaDefinitions = cqg_qformula_definitions;

      // Fill up new data
      ClearAllData();

      long count;
      hr = cqg_qformula_definitions->get_Count(&count);
      AssertCOMError<ICQGQFormulaDefinitions>(cqg_qformula_definitions, hr);

      if (count > 0)
      {
         for (long i = 0; i < count; ++i)
         {
            ATL::CComPtr<ICQGQFormulaDefinition> spQFormulaDefinition;
            hr = cqg_qformula_definitions->get_Item(i, &spQFormulaDefinition);
            AssertCOMError<ICQGQFormulaDefinitions>(cqg_qformula_definitions, hr);

            ATL::CComBSTR name;
            hr = spQFormulaDefinition->get_Name(&name);
            AssertCOMError(spQFormulaDefinition, hr);

            const int row = m_cmbDefinition.AddString(ATL::CW2T(name));
            m_cmbDefinition.SetItemData(row, i);
         }

         // Select first definition
         m_cmbDefinition.SetCurSel(0);
         OnCbnSelchangeCmbDefinition();
      }
   }
   catch (const std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}