﻿using System;
using System.Windows.Forms;
using CQG;

namespace OrderPlacingSample
{
   internal sealed class AuxiliaryMethods
   {
      /// <summary>
      /// Returns display format of specified price if it is valid, otherwise N/A.
      /// </summary>
      /// <param name="instrument">CQGInstrument object which is used to get display format.</param>
      /// <param name="price">Raw price.</param>
      /// <returns>String representation of specified price.</returns>
      public static string GetDisplayPrice(CQGCEL CEL, CQGInstrument instrument, double price)
      {
         if (instrument != null && CEL.IsValid(price))
         {
            return instrument.ToDisplayPrice(price);
         }

         return Resources.Resource.N_A;
      }

      /// <summary>
      /// Returns the valid price of the selected instrument.
      /// </summary>
      /// <param name="instrument">CQGInstrument object which is used to get valid price.</param>
      /// <returns>Valid price of the selected instrument.</returns>
      public static string GetValidPrice(CQGInstrument instrument)
      {
         if (instrument == null)
         {
            return Resources.Resource.N_A;
         }
         
         if (instrument.Trade.IsValid)
         {
            return instrument.ToDisplayPrice(instrument.Trade.Price);
         }
         if (instrument.Ask.IsValid)
         {
            return instrument.ToDisplayPrice(instrument.Ask.Price);
         }
         if (instrument.Bid.IsValid)
         {
            return instrument.ToDisplayPrice(instrument.Bid.Price);
         }

         return Resources.Resource.N_A;
      }

      /// <summary>
      /// Returns increment/decrement based on scroll click.
      /// </summary>
      /// <param name="e">Scroll bar change event argument.</param>
      /// <returns>Increment/decrement based on scroll click.</returns>
      public static int UpDownSwitch(ScrollEventArgs e)
      {
         int upDown = 0;
         switch (e.Type)
         {
            case ScrollEventType.SmallIncrement:
            case ScrollEventType.LargeIncrement:
               upDown = -1;
               break;
            case ScrollEventType.SmallDecrement:
            case ScrollEventType.LargeDecrement:
               upDown = 1;
               break;
         }

         return upDown;
      }

      /// <summary>
      /// Increments/decrements price in specified control by specified instrument tick size
      /// when scroll bar is clicked.
      /// </summary>
      /// <param name="instrument">CQGInstrument object.</param>
      /// <param name="priceControl">TextBox control which contains current price.</param>
      /// <param name="e">Scroll bar change event argument.</param>
      public static void UpDownPrice(CQGInstrument instrument,
                                       TextBox priceControl,
                                       ScrollEventArgs e)
      {
         if (instrument == null || priceControl.Text == Resources.Resource.N_A)
         {
            return;
         }

         int upDown = UpDownSwitch(e);

         // Increase/decrease price by tick
         if (upDown != 0)
         {
            double rawPrice = instrument.FromDisplayPrice(priceControl.Text);
            bool isVariableTickSize = (bool)instrument.Properties[eInstrumentProperty.ipIsVariableTickSize].Value;
            rawPrice += upDown * (isVariableTickSize ? instrument.GetTickSizeFor(rawPrice) : instrument.TickSize);
            priceControl.Text = instrument.ToDisplayPrice(rawPrice);
         }
      }

      /// <summary>
      /// Increments/decrements tick in specified control, when scroll bar is clicked.
      /// </summary>
      /// <param name="tickControl">TextBox control which contains current tick.</param>
      /// <param name="e">Scroll bar change event argument.</param>
      public static void UpDownTick(TextBox tickControl, ScrollEventArgs e)
      {
         int upDown = UpDownSwitch(e);

         tickControl.Text = (int.Parse(tickControl.Text) + upDown).ToString();
      }
   }
}

