// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include "Common.h"
#include "OrderPlacing.h"
#include "OrderPlacingDlg.h"

namespace
{
   /// \brief Shows message box with symbol resolution failed reason.
   /// \param symbol - Requested symbol
   /// \param cqg_error - Error occured during a symbol resolution process
   void onSymbolResolutonFailed(BSTR symbol, ICQGError* cqg_error)
   {
      ATL::CComBSTR errDescription;
      auto hr = cqg_error->get_Description(&errDescription);
      AssertCOMError<ICQGError>(cqg_error, hr);

      CStringW msg;
      msg.AppendFormat(L"%s resolution failed. %s", symbol, errDescription.m_str);
      AfxMessageBox(ATL::CW2T(msg), MB_ICONWARNING);
   }
}

STDMETHODIMP COrderPlacingDlg::OnDataError(
                                 LPDISPATCH /*obj*/,
                                 BSTR errorDescription)
{
   CString errMsg("Data error: ");
   errMsg += ATL::CW2T(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP COrderPlacingDlg::OnGWConnectionStatusChanged(
                                 eConnectionStatus newStatus)
{
   try
   {
      m_lblGWConnection.SetWindowText(_T("GW connection : ") + ConnectionStatusToString(newStatus));

      if (newStatus == csConnectionUp)
      {
         m_spCQGCEL->put_AccountSubscriptionLevel(aslAccountUpdatesAndOrders);
         m_GWConnectionUp = true;
      }
      else
      {
         m_GWConnectionUp = false;
      }
      
      EnableControls();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP COrderPlacingDlg::OnDataConnectionStatusChanged(
                                 eConnectionStatus newStatus)
{
   try
   {
      CString status;
      
      switch (newStatus)
      {
         case csConnectionDelayed:
            status = _T("delayed");
            break;
         case csConnectionDown:
            status = _T("down");
            break;
         case csConnectionUp:
            status = _T("up");
            break;
         default:
            assert(!"Unexpected data connection status");
      }
      m_lblDataConnection.SetWindowText(_T("Data connection ") + status);

      m_DataConnectionUp = (newStatus == csConnectionUp);
   
      EnableControls();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP COrderPlacingDlg::OnAccountChanged(
                                 eAccountChangeType change, 
                                 ICQGAccount* /*account*/, 
                                 ICQGPosition* /*position*/)
{
   try
   {
      if (change == actAccountsReloaded)
      {
         ReloadAccounts();
      }
   }
   catch (std::exception& e) 
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP COrderPlacingDlg::OnInstrumentResolved(
   BSTR symbol,
   ICQGInstrument* instrument,
   ICQGError* cqg_error)
{
   try
   {
      VARIANT_BOOL errorOccured = VARIANT_FALSE;
      auto hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &errorOccured);
      AssertCOMError<ICQGError>(cqg_error, hr);

      if (errorOccured)
      {
         onSymbolResolutonFailed(symbol, cqg_error);
         return S_OK;
      }

      ATL::CComBSTR fullName;
      hr = instrument->get_FullName(&fullName);
      AssertCOMError<ICQGInstrument>(instrument, hr);

      // If instrument is already subscribed do not add it to the list
      if (m_lsInstruments.FindStringExact(0, ATL::CW2T(fullName)) == LB_ERR)
      {
         m_lsInstruments.AddString(ATL::CW2T(fullName));
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
   return S_OK;
}

STDMETHODIMP COrderPlacingDlg::OnInstrumentChanged(
                                 ICQGInstrument* instrument, 
                                 ICQGQuotes* /*quotes*/, 
                                 ICQGInstrumentProperties* /*props*/)
{
   try
   {
      // Get selected instrument
      ATL::CComPtr<ICQGInstrument> spSelInstrument = GetSelectedInstrument();

      if (spSelInstrument == NULL)
      {
         // No instrument is selected
         return S_OK;
      }

      // Get changed instrument's full name
      ATL::CComBSTR fullName;
      HRESULT hr = instrument->get_FullName(&fullName);
      AssertCOMError<ICQGInstrument>(instrument, hr);

      // Get selected instrument's full name
      ATL::CComBSTR selFullName;
      hr = spSelInstrument->get_FullName(&selFullName);
      AssertCOMError(spSelInstrument, hr);
   
      if (fullName == selFullName)
      {
         // Selected instrument was changed, show it's quotes
         ShowQuotes(instrument);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP COrderPlacingDlg::OnOrderChanged(
                                 eChangeType change, 
                                 ICQGOrder* order, 
                                 ICQGOrderProperties* /*oldProperties*/, 
                                 ICQGFill* /*fill*/, 
                                 ICQGError* /*cqgerr*/)
{
   try
   {
      // Get order's account
      ATL::CComPtr<ICQGAccount> spAccount;
      HRESULT hr = order->get_Account(&spAccount);
      AssertCOMError<ICQGOrder>(order, hr);

      // Get order's account ID 
      long orderAccID;
      hr = spAccount->get_GWAccountID(&orderAccID);
      AssertCOMError(spAccount, hr);

      // Get selected account ID
      long selAccID = static_cast<long>(m_cmbAccounts.GetItemData(m_cmbAccounts.GetCurSel()));

      if (orderAccID != selAccID)
      {
         // Order's account is not selected
         return S_OK;
      }

      // Get order status
      eOrderStatus orderStatus;
      hr = order->get_GWStatus(&orderStatus);
      AssertCOMError<ICQGOrder>(order, hr);

      // Get filter type
      int status = FilterTypeToOrderStatus(static_cast<eFilterType>(m_tabOrders.GetCurSel()));

      if (status == m_INVALID_INTEGER || status == orderStatus) 
      {
         // Add or update order in the list control
         ShowOrder(order, change);
      }
      else if (GetOrderRow(order) > -1)
      {
         // If order exist in the list control, but status changed, remove it from the list
         ShowOrder(order, ctRemoved);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}
