﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CQG;

namespace OrderPlacingSample
{
   public partial class frmBracketProperties : Form
   {
      /// <summary>
      /// CQGCEL object.
      /// </summary>
      private CQGCEL m_CEL;

      /// <summary>
      /// CQGOrder - main order.
      /// </summary>
      private CQGOrder m_MainOrder;

      /// <summary>
      /// Main order side.
      /// </summary>
      private eOrderSide m_MainOrderSide;

      #region Auxiliary methods

      /// <summary>
      /// Initializes bracket order properties form.
      /// </summary>
      /// <param name="cel">CQGCEL object to be used to create bracket order.</param>
      /// <param name="mainOrder">CQGOrder object to be used as main order.</param>
      /// <param name="mainOrderSide">Main order side</param>
      public frmBracketProperties(CQGCEL cel, CQGOrder mainOrder, eOrderSide mainOrderSide)
      {
         try
         {
            // Required for Windows Form Designer support
            InitializeComponent();

            m_CEL = cel;
            m_MainOrder = mainOrder;
            m_MainOrderSide = mainOrderSide;

            InitializeSubelements(mainOrderSide);
         }
         catch (System.Exception ex)
         {
            throw new TypeInitializationException("frmBracketProperties", ex);
         }
      }

      /// <summary>
      /// Gets price from main order.
      /// </summary>
      private double GetPrice()
      {
         switch (m_MainOrder.Type)
         {
            case eOrderType.otStop:
               return m_MainOrder.StopPrice;
            case eOrderType.otLimit:
            case eOrderType.otStopLimit:
               return m_MainOrder.LimitPrice;
            case eOrderType.otMarket:
               return (m_MainOrderSide == eOrderSide.osdBuy ? m_MainOrder.Instrument.Bid.Price : m_MainOrder.Instrument.Ask.Price);

            default: throw new ArgumentException("getPrice");
         }
      }

      /// <summary>
      /// Initializes summary information line of main order.
      /// </summary>
      /// <param name="mainOrderSide">Main order side</param>
      private string CreateInfoLine(eOrderSide mainOrderSide)
      {
         string infoLine = string.Format("{0} {1} {2} @ {3} BRKT {4}",
            /*0=*/   (mainOrderSide == eOrderSide.osdBuy ? "BUY" : "SELL"),
            /*1=*/   Math.Abs(m_MainOrder.QuantityFractional),
            /*2=*/   m_MainOrder.InstrumentName,
            /*3=*/   GetPrice(),
            /*4=*/   m_MainOrder.Type.ToString());

         return infoLine;
      }

      /// <summary>
      /// Initializes bracket order properties controls.
      /// </summary>
      /// <param name="mainOrderSide">Main order side</param>
      private void InitializeSubelements(eOrderSide mainOrderSide)
      {
         Text = "Confirm";

         lblMainInfo.Text = CreateInfoLine(mainOrderSide);

         btnPlace.Text = "Place " + (mainOrderSide == eOrderSide.osdBuy ? "BUY" : "SELL") + " Order";
         btnPlace.ForeColor = mainOrderSide == eOrderSide.osdBuy ? Color.FromArgb(0, 192, 0) : Color.FromArgb(192, 0, 0);

         InitializeSpeculationTypesTypes();
         InitializeTargetOffset();
         InitializeStopLossOffset();
         InitializeOrderTypes();
      }

      /// <summary>
      /// Initializes main order speculation types combo control.
      /// </summary>
      private void InitializeSpeculationTypesTypes()
      {
         foreach (var speculationType in Enum.GetValues(typeof(eSpeculationType)))
         {
            cmbSpeculationTypes.Items.Add(speculationType);
         }
         cmbSpeculationTypes.SelectedIndex = 0;
      }

      /// <summary>
      /// Initializes target order offset types combo control.
      /// </summary>
      private void InitializeTargetOffset()
      {
         cmbTargetOffset.Items.Add(eOffset.Ticks);
         cmbTargetOffset.Items.Add(eOffset.Price);
         cmbTargetOffset.SelectedIndex = 0;
      }

      /// <summary>
      /// Initializes stop-loss order offset types combo control.
      /// </summary>
      private void InitializeStopLossOffset()
      {
         cmbStopLossOffset.Items.Add(eOffset.Ticks);
         cmbStopLossOffset.Items.Add(eOffset.Price);
         cmbStopLossOffset.SelectedIndex = 0;

         lblStopLossTickOrPrice.Text = eOffset.Ticks.ToString();
      }

      /// <summary>
      /// Initializes stop-loss order types combo control.
      /// </summary>
      private void InitializeOrderTypes()
      {
         cmbStopLossOrderTypes.Items.Add(eOrderType.otStopLimit);
         cmbStopLossOrderTypes.Items.Add(eOrderType.otStop);
         cmbStopLossOrderTypes.SelectedIndex = 0;
      }

      /// <summary>
      /// Cancels bracket properties form.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data.</param>
      private void btnCancel_Click(object sender, EventArgs e)
      {
         Close();
      }

      #endregion

      #region Form controls event handlers

      /// <summary>
      /// Gathers values set in GUI and places bracket order.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void btnPlace_Click(object sender, EventArgs e)
      {
         CQGBracketOrder bracketOrder = m_CEL.CreateBracketOrder(m_MainOrder, (eOrderType)cmbStopLossOrderTypes.SelectedItem);
         m_MainOrder.Properties[eOrderProperty.opSpeculationType].Value = (eSpeculationType)cmbSpeculationTypes.SelectedItem;

         bracketOrder.IncludeTargetOrder = chkIncludeTarget.Checked;
         bracketOrder.IncludeStopLossOrder = chkIncludeStopLoss.Checked;

         try
         {
            if (bracketOrder.IncludeTargetOrder)
            {
               if ((eOffset)cmbTargetOffset.SelectedItem == eOffset.Ticks)
               {
                  bracketOrder.TargetLeg.LimitOffset = int.Parse(txtTargetTickPrice.Text);
               }
               else
               {
                  bracketOrder.TargetLeg.LimitPrice = m_MainOrder.Instrument.FromDisplayPrice(txtTargetTickPrice.Text);
               }
            }

            if (bracketOrder.IncludeStopLossOrder)
            {
               if ((eOffset)cmbStopLossOffset.SelectedItem == eOffset.Ticks)
               {
                  bracketOrder.StopLossLeg.StopOffset = int.Parse(txtStopLossStopTickPrice.Text);
               }
               else
               {
                  bracketOrder.StopLossLeg.StopPrice = m_MainOrder.Instrument.FromDisplayPrice(txtStopLossStopTickPrice.Text);
               }

               if (bracketOrder.StopLossLeg.OrderType == eOrderType.otStopLimit)
               {
                  if ((eOffset)cmbStopLossOffset.SelectedItem == eOffset.Ticks)
                  {
                     bracketOrder.StopLossLeg.LimitOffset = int.Parse(txtStopLossLimitTickPrice.Text);
                  }
                  else
                  {
                     bracketOrder.StopLossLeg.LimitPrice = m_MainOrder.Instrument.FromDisplayPrice(txtStopLossLimitTickPrice.Text);
                  }
               }
            }

            bracketOrder.Place();
            Close();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmBracketProperties", "btnPlace_Click", ex);
         }
      }

      /// <summary>
      /// Sets target price/tick based on changed value.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void cmbTargetOffset_SelectedIndexChanged(object sender, EventArgs e)
      {
         if ((eOffset)cmbTargetOffset.SelectedItem == eOffset.Ticks)
         {
            txtTargetTickPrice.Text = "1";
         }
         else
         {
            txtTargetTickPrice.Text = AuxiliaryMethods.GetValidPrice(m_MainOrder.Instrument);
         }
      }

      /// <summary>
      /// Sets stop-loss price/tick based on changed value.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void cmbStopLossOffset_SelectedIndexChanged(object sender, EventArgs e)
      {
         if ((eOffset)cmbStopLossOffset.SelectedItem == eOffset.Ticks)
         {
            txtStopLossStopTickPrice.Text = "1";
            txtStopLossLimitTickPrice.Text = "1";
            lblStopLossTickOrPrice.Text = eOffset.Ticks.ToString();
         }
         else
         {
            txtStopLossStopTickPrice.Text = AuxiliaryMethods.GetValidPrice(m_MainOrder.Instrument);
            txtStopLossLimitTickPrice.Text = AuxiliaryMethods.GetValidPrice(m_MainOrder.Instrument);
            lblStopLossTickOrPrice.Text = eOffset.Price.ToString();
         }
      }

      /// <summary>
      /// Enables/disables target order controls based on CheckBox.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void chkIncludeTarget_CheckedChanged(object sender, EventArgs e)
      {
         txtTargetTickPrice.Visible = chkIncludeTarget.Checked;
         cmbTargetOffset.Visible = chkIncludeTarget.Checked;
         vsbTargetTickPrice.Visible = chkIncludeTarget.Checked;
      }

      /// <summary>
      /// Enables/disables stop-loss order controls based on CheckBox.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void chkIncludeStopLoss_CheckedChanged(object sender, EventArgs e)
      {
         cmbStopLossOffset.Visible = chkIncludeStopLoss.Checked;
         txtStopLossStopTickPrice.Visible = chkIncludeStopLoss.Checked;
         cmbStopLossOrderTypes.Visible = chkIncludeStopLoss.Checked;
         vsbStopLossStopTickPrice.Visible = chkIncludeStopLoss.Checked;

         if ((eOrderType)cmbStopLossOrderTypes.SelectedItem == eOrderType.otStopLimit)
         {
            txtStopLossLimitTickPrice.Visible = chkIncludeStopLoss.Checked;
            vsbStopLossLimitTickPrice.Visible = chkIncludeStopLoss.Checked;
            lblStopLossTickOrPrice.Visible = chkIncludeStopLoss.Checked;
         }
      }

      /// <summary>
      /// Enables/disables stop-loss limit controls based on selected stop-loss order type.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void cmbStopLossOrderTypes_SelectedIndexChanged(object sender, EventArgs e)
      {
         var val = (eOrderType)cmbStopLossOrderTypes.SelectedItem == eOrderType.otStopLimit;
         txtStopLossLimitTickPrice.Visible = val;
         vsbStopLossLimitTickPrice.Visible = val;
         lblStopLossTickOrPrice.Visible = val;
      }

      /// <summary>
      /// Increments/decrements target order tick/price value.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A ScrollEventArgs that contains the event data.</param>
      private void vsbTargetTickPrice_Scroll(object sender, ScrollEventArgs e)
      {
         try
         {
            if ((eOffset)cmbTargetOffset.SelectedItem == eOffset.Price)
            {
               AuxiliaryMethods.UpDownPrice(m_MainOrder.Instrument, txtTargetTickPrice, e);
            }
            else
            {
               AuxiliaryMethods.UpDownTick(txtTargetTickPrice, e);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmBracketProperties", "vsbTargetTickPrice_Scroll", ex);
         }
      }

      /// <summary>
      /// Increments/decrements stop-loss stop order tick/price value.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A ScrollEventArgs that contains the event data.</param>
      private void vsbStopLossStopTickPrice_Scroll(object sender, ScrollEventArgs e)
      {
         try
         {
            if ((eOffset)cmbStopLossOffset.SelectedItem == eOffset.Price)
            {
               AuxiliaryMethods.UpDownPrice(m_MainOrder.Instrument, txtStopLossStopTickPrice, e);
            }
            else
            {
               AuxiliaryMethods.UpDownTick(txtStopLossStopTickPrice, e);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmBracketProperties", "vsbStopLossStopTickPrice_Scroll", ex);
         }
      }

      /// <summary>
      /// Increments/decrements stop-loss limit order tick/price value.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A ScrollEventArgs that contains the event data.</param>
      private void vsbStopLossLimitTickPrice_Scroll(object sender, ScrollEventArgs e)
      {
         try
         {
            if ((eOffset)cmbStopLossOffset.SelectedItem == eOffset.Price)
            {
               AuxiliaryMethods.UpDownPrice(m_MainOrder.Instrument, txtStopLossLimitTickPrice, e);
            }
            else
            {
               AuxiliaryMethods.UpDownTick(txtStopLossLimitTickPrice, e);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmBracketProperties", "vsbStopLossLimitTickPrice_Scroll", ex);
         }
      }

      #endregion

      /// <summary>
      /// Specifies offset types.
      /// </summary>
      private enum eOffset
      {
         Ticks = 1,
         Price
      }
   }
}
