using System;
using System.Windows.Forms;
using CQG;

namespace OrderPlacingSample
{
   /// <summary>
   /// This form is used to modify specified order properties.
   /// </summary>
   public partial class frmModify : Form
   {
      /// <summary>
      /// CQGCEL object.
      /// </summary>
      private CQGCEL m_CEL;

      /// <summary>
      /// CQGOrder object to modify.
      /// </summary>
      private CQGOrder m_Order;

      /// <summary>
      /// CQGOrderModify object used to modify order's parameters.
      /// </summary>
      private CQGOrderModify m_OrderModify;

      /// <summary>
      /// Initializes modify controls.
      /// </summary>
      /// <param name="cel">CQGCEL object.</param>
      /// <param name="order">CQGOrder object to be modified.</param>
      public frmModify(CQGCEL cel, CQGOrder order)
      {
         try
         {
            // Required for Windows Form Designer support
            InitializeComponent();

            this.Text = "Modify order : " + order.GWOrderID;

            m_CEL = cel;
            m_Order = order;
            m_OrderModify = order.PrepareModify();

            InitializePricesModify();
            InitializeQuantityModify();
         }
         catch (System.Exception ex)
         {
            throw new TypeInitializationException("frmModify", ex);
         }
      }

      #region Auxiliary methods

      /// <summary>
      /// Initializes controls used to modify order's quantity.
      /// </summary>
      private void InitializeQuantityModify()
      {
         nudQuantity.Value = Math.Abs(m_Order.QuantityFractional);
         nudQuantity.Enabled = (m_OrderModify.Properties[eOrderProperty.opQuantityFractional] != null);
      }

      /// <summary>
      /// Initializes controls used to modify order's prices.
      /// </summary>
      private void InitializePricesModify()
      {
         CQGInstrument instrument = m_Order.Instrument;

         txtLimitPrice.Text = Resources.Resource.N_A;
         txtLimitPrice.Enabled = false;
         vsbLimitPrice.Enabled = false;

         txtStopPrice.Text = Resources.Resource.N_A;
         txtStopPrice.Enabled = false;
         vsbStopPrice.Enabled = false;

         if (instrument != null)
         {
            if (m_OrderModify.Properties[eOrderProperty.opLimitPrice] != null)
            {
               txtLimitPrice.Text = instrument.ToDisplayPrice(m_Order.LimitPrice);
               txtLimitPrice.Enabled = true;
               vsbLimitPrice.Enabled = true;
            }

            if (m_OrderModify.Properties[eOrderProperty.opStopPrice] != null)
            {
               txtStopPrice.Text = instrument.ToDisplayPrice(m_Order.StopPrice);
               txtStopPrice.Enabled = true;
               vsbStopPrice.Enabled = true;
            }
         }
      }


      #endregion

      #region Form controls event handlers

      /// <summary>
      /// Gathers modified values and modifies order.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data.</param>
      private void btnOK_Click(object sender, EventArgs e)
      {
         try
         {
            bool isModified = false;

            // Change quantity if allowed
            CQGOrderProperty property = m_OrderModify.Properties[eOrderProperty.opQuantityFractional];
            if (property != null)
            {
               var quantity = nudQuantity.Value;
               // Checking order side by the order's quanitity sign
               quantity *= Math.Sign(m_Order.QuantityFractional);
               if (quantity != m_Order.QuantityFractional)
               {
                  property.Value = quantity;
                  isModified = true;
               }
            }

            // Change limit price if allowed
            property = m_OrderModify.Properties[eOrderProperty.opLimitPrice];
            if (property != null && txtLimitPrice.Text != Resources.Resource.N_A)
            {
               double limitPrice = m_Order.Instrument.FromDisplayPrice(txtLimitPrice.Text);
               if (limitPrice != m_Order.LimitPrice)
               {
                  property.Value = limitPrice;
                  isModified = true;
               }
            }

            // Change stop price if allowed
            property = m_OrderModify.Properties[eOrderProperty.opStopPrice];
            if (property != null && txtStopPrice.Text != Resources.Resource.N_A)
            {
               double stopPrice = m_Order.Instrument.FromDisplayPrice(txtStopPrice.Text);
               if (stopPrice != m_Order.StopPrice)
               {
                  property.Value = stopPrice;
                  isModified = true;
               }
            }

            // Modify order if needed
            if (isModified)
            {
               m_Order.Modify(m_OrderModify);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.ShowErrorMessage(ex.Message, "Order modification", this);
         }
         finally
         {
            this.Close();
         }
      }

      /// <summary>
      /// Increments/Decrements limit price.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A ScrollEventArgs that contains the event data. </param>
      private void vsbLimitPrice_Scroll(object sender, ScrollEventArgs e)
      {
         try
         {
            AuxiliaryMethods.UpDownPrice(m_Order.Instrument, txtLimitPrice, e);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmModify", "vsbLimitPrice_Scroll", ex);
         }
      }

      /// <summary>
      /// Increments/Decrements stop price.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A ScrollEventArgs that contains the event data. </param>
      private void vsbStopPrice_Scroll(object sender, ScrollEventArgs e)
      {
         try
         {
            AuxiliaryMethods.UpDownPrice(m_Order.Instrument, txtStopPrice, e);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmModify", "vsbStopPrice_Scroll", ex);
         }
      }


      #endregion
   }
}
