using System;
using System.Collections;
using System.Windows.Forms;
using CQG;

namespace OrderPlacingSample
{
   /// <summary>
   /// This form shows instrument quotes and allows to place and manage orders.
   /// </summary>
   public partial class frmOrderPlacing : Form
   {
      /// <summary>
      /// Custom short date format
      /// </summary>
      internal const string DATE_FORMAT_SHORT = "MM/dd/yyyy";

      /// <summary>
      /// Custom long date format
      /// </summary>
      internal const string DATE_FORMAT_LONG = "MM/dd/yyyy HH:mm:ss";

      /// <summary>
      /// Custom time format
      /// </summary>
      internal const string TIME_FORMAT = "HH:mm:ss";

      /// <summary>
      /// "Manual Fills" string constant.
      /// </summary>
      private const string MANUAL_FILLS = "Manual Fills";

      /// <summary>
      /// "Buy" string constant.
      /// </summary>
      private const string ORDER_SIDE_BUY = "Buy";

      /// <summary>
      /// "Sell" string constant.
      /// </summary>
      private const string ORDER_SIDE_SELL = "Sell";

      /// <summary>
      /// CQG API instance (CQGCEL).
      /// </summary>
      private CQGCEL m_CEL;

      /// <summary>
      /// Hash table keeps mapping of account descriptive name to account GWID.
      /// </summary>
      private readonly Hashtable m_AccountsMap = new Hashtable(20);

      /// <summary>
      /// Flag to show whether the "Manual Fills Mode" is on or off.
      /// </summary>
      private bool m_IsManualFillMode = false;

      /// <summary>
      /// Form constructor.
      /// </summary>
      public frmOrderPlacing()
      {
         try
         {
            // Required for Windows Form Designer support
            InitializeComponent();

            InitializeOrdersList();
            InitializeManualFillsList();
            InitializeOrderTypes();
            InitializeDurationTypes();
            InitializeOpenCloseInstructions();

            SetOrderPrices(null, eOrderType.otUndefined);

            ClearInstrumentData();

            // Initializes connection controls
            cel_DataConnectionStatusChanged(eConnectionStatus.csConnectionDown);
            cel_GWConnectionStatusChanged(eConnectionStatus.csConnectionDown);

            // Disable buttons
            EnableButtons();
         }
         catch (System.Exception ex)
         {
            throw new TypeInitializationException("frmOrderPlacing", ex);
         }
      }

      #region CQGCEL event handlers

      /// <summary> 
      /// This event is fired, when some changes occur in the connection 
      /// with the CQG data server.
      /// </summary>
      /// <param name="newStatus"> The current status of the 
      /// connection with the Data server.</param>
      private void cel_DataConnectionStatusChanged(eConnectionStatus newStatus)
      {
         try
         {
            string status;
            System.Drawing.Color backColor;

            switch (newStatus)
            {
               case eConnectionStatus.csConnectionUp:
                  backColor = System.Drawing.Color.FromArgb(192, 209, 205);
                  status = "Up";
                  break;
               case eConnectionStatus.csConnectionDelayed:
                  backColor = System.Drawing.Color.FromArgb(255, 114, 0);
                  status = "Delayed";
                  break;
               case eConnectionStatus.csConnectionDown:
                  backColor = System.Drawing.Color.FromArgb(255, 114, 0);
                  status = "Down";
                  break;
               default:
                  throw new ArgumentException("Unknown connection status.", "newStatus");
            }

            lblDataConnection.BackColor = backColor;
            lblDataConnection.Text = "Data Connection is " + status;

            EnableDataControls(newStatus == eConnectionStatus.csConnectionUp);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "cel_DataConnectionStatusChanged", ex);
         }
      }

      /// <summary> 
      /// This event is fired, when some changes occur in the connection 
      /// with the CQG Gateway server.
      /// </summary>
      /// <param name="newStatus"> The current status of the 
      /// connection with the Gateway server.</param>
      private void cel_GWConnectionStatusChanged(eConnectionStatus newStatus)
      {
         try
         {
            System.Drawing.Color backColor;

            if (newStatus == eConnectionStatus.csConnectionUp)
            {
               m_CEL.AccountSubscriptionLevel = eAccountSubscriptionLevel.aslAccountUpdatesAndOrders;
               backColor = System.Drawing.Color.FromArgb(192, 209, 205);
            }
            else
            {
               backColor = System.Drawing.Color.FromArgb(255, 114, 0);
            }

            lblGWConnection.BackColor = backColor;
            lblGWConnection.Text = "GW Connection : " + ConnectionStatusToString(newStatus);

            // Disable controls until accounts are reloaded
            EnableGWControls(false);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "cel_GWConnectionStatusChanged", ex);
         }
      }

      /// <summary>
      /// This event is fired, when CQGCEL detects some abnormal discrepancy between data expected and data received.
      /// </summary>
      /// <param name="obj">Object, in which the error occurred.</param>
      /// <param name="errorDescription">String, describing the error.</param>
      private void cel_DataError(object obj, string errorDescription)
      {
         CQGError cqgErr = obj as CQGError;
         if (m_CEL.IsValid(cqgErr))
         {
            if (cqgErr.Code == 102)
            {
               errorDescription += " Restart the application.";
            }
            else if (cqgErr.Code == 125)
            {
               errorDescription += " Turn on CQG Client and restart the application.";
            }
         }

         MessageBox.Show(errorDescription, "Data Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }

      /// <summary>
      /// This event is fired when the account or position information is changed.
      /// </summary>
      /// <param name="change">Account change type which allows to differentiate the occurred changes.</param>
      /// <param name="account">A CQGAccount object representing the account 
      /// to which the current change refers.</param>
      /// <param name="position">A CQGPosition object representing the position 
      /// to which the current change refers.</param>
      private void cel_AccountChanged(eAccountChangeType change,
                                      CQGAccount account,
                                      CQGPosition position)
      {
         try
         {
            if (change == eAccountChangeType.actAccountsReloaded)
            {
               foreach (CQGAccount acc in m_CEL.Accounts)
               {
                  m_CEL.RequestManualFills(acc.GWAccountID, eManualFillsDataLevel.mfdlSnapshotAndUpdates);
               }

               FillAccountsList();
               EnableGWControls(true);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "cel_AccountChanged", ex);
         }
      }

      /// <summary>
      /// This event is fired when a new instrument is resolved and subscribed or resolution failed.
      /// </summary>
      /// <param name="symbol">The commodity symbol that was requested by user in NewInstrument method.</param>
      /// <param name="instrument">Subscribed CQGInstrument object.</param>
      /// <param name="cqgError">Error happened during a resolution process.</param>
      private void cel_InstrumentResolved(string symbol, CQGInstrument cqgInstrument, CQGError cqgError)
      {
         try
         {
            if (m_CEL.IsValid(cqgError))
            {
               ErrorHandler.HandleError("frmOrderPlacing", "cel_InstrumentResolved", cqgError);
               return;
            }

            instrumentSubscribed(symbol, cqgInstrument);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "CEL_InstrumentResolved", ex);
         }
      }

      /// <summary>
      /// The method called when a new instrument is resolved and subscribed.
      /// </summary>
      /// <param name="symbl">The commodity symbol that was requested by user in NewInstrument method.</param>
      /// <param name="instrument">Subscribed CQGInstrument object.</param>
      private void instrumentSubscribed(string symbl, CQGInstrument instrument)
      {
         if (lstInstruments.Items.Contains(instrument.FullName))
         {
            return;
         }

         int index = lstInstruments.Items.Add(instrument.FullName);
         lstInstruments.SelectedIndex = index;
      }

      /// <summary>
      /// This event is fired when any of the instrument quotes or dynamic instrument properties are changed.
      /// </summary>
      /// <param name="instrument">Changed instrument.</param>
      /// <param name="quotes">Collection of changed quotes.</param>
      /// <param name="props">Collection of changed dynamic properties. </param>
      private void cel_InstrumentChanged(CQGInstrument instrument,
                                         CQGQuotes quotes,
                                         CQGInstrumentProperties props)
      {
         try
         {
            CQGInstrument selInstrument = GetSelectedInstrument();
            if (selInstrument == null)
            {
               return;
            }

            if (selInstrument.FullName == instrument.FullName)
            {
               DisplayInstrumentProperties(instrument);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "cel_InstrumentChanged", ex);
         }
      }

      /// <summary>
      /// This event is fired when a new order is added to the collection, 
      /// order status is changed or the order is removed from the collection.
      /// </summary>
      /// <param name="change">The type of the occurred change.</param>
      /// <param name="order">CQGOrder object representing the order to which the change refers.</param>
      /// <param name="oldProperties">CQGOrderProperties collection representing the old 
      /// values of the changed order properties.</param>
      /// <param name="fill">CQGFill object representing the last fill of the order.</param>
      /// <param name="objError">CQGError object representing either an error sent by CQG Gateway 
      /// or an intermediate error.</param>
      private void cel_OrderChanged(eChangeType change,
                                    CQGOrder order,
                                    CQGOrderProperties oldProperties,
                                    CQGFill fill,
                                    CQGError objError)
      {
         try
         {
            if (!IsOrdersTab())
            {
               return;
            }

            // Get current selected account
            CQGAccount account = GetSelectedAccount();
            if (account == null)
            {
               return;
            }

            // Check if the order belongs to selected account
            if (account.InternalOrders[order.GWOrderID] != null)
            {
               DisplayOrder(change, order);
            }

            EnableButtons();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "cel_OrderChanged", ex);
         }
      }

      /// <summary>
      /// This event is fired once every second to synchronize with the exchange.
      /// </summary>
      /// <param name="dateTime">Date containing the line time (time on the current exchange) 
      /// converted to the timezone.</param>
      private void cel_LineTimeChanged(DateTime dateTime)
      {
         lblLineTime.Text = dateTime.ToString(DATE_FORMAT_LONG);
      }

      /// <summary>
      /// This event is fired when the list of manual fills request is resolved or when some error has occurred during manual fills request processing.
      /// </summary>
      /// <param name="cqg_manual_fills">Specifies the collection of manual fills. If an error has occurred, the collection is empty.</param>
      /// <param name="cqg_error">Specifies the CQGError object describing the last error occurred during the processing of manual fills request or 
      /// Nothing/Invalid_Error in case of no error.
      /// CQGCEL.IsValid(Invalid_Error) returns False.</param>
      private void cel_ManualFillsResolved(CQGManualFills cqg_manual_fills, CQGError cqg_error)
      {
         try
         {
            if (IsOrdersTab())
            {
               return;
            }

            if (m_CEL.IsValid(cqg_error))
            {
               ErrorHandler.HandleError("frmOrderPlacing", "cel_ManualFillsResolved", cqg_error);
               return;
            }

            DisplayManualFills();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "cel_ManualFillsResolved", ex);
         }
      }

      /// <summary>
      /// This event is fired when a new manual fill was added, some manual fill properties were changed, 
      /// or the manual fill was removed only if subscription data level for appropriate request was requested with updates notifications.
      /// </summary>
      /// <param name="cqg_manual_fill">Specifies the changed manual fill instance, represented as CQGManualFill.</param>
      /// <param name="modify_type">Manual fill modify type which allows to differentiate the occurred modifies.</param>
      private void cel_ManualFillChanged(CQGManualFill cqg_manual_fill, eManualFillUpdateType modify_type)
      {
         try
         {
            if (IsOrdersTab())
            {
               return;
            }

            // Get current selected account
            CQGAccount account = GetSelectedAccount();
            if (account == null)
            {
               return;
            }

            if (modify_type == eManualFillUpdateType.mfutDelete)
            {
               ListViewItem foundItem = null;
               // Find item in the list
               foreach (ListViewItem item in lsvManualFills.Items)
               {
                  if (item.Text == cqg_manual_fill.TradeId.ToString())
                  {
                     foundItem = item;
                     break;
                  }
               }

               if (foundItem != null)
               {
                  lsvManualFills.Items.Remove(foundItem);
               }
            }
            else
            {
               // Check if the manual fill belongs to selected account
               if (m_CEL.AllManualFills[account.GWAccountID][cqg_manual_fill.TradeId] != null)
               {
                  DisplayManualFill(modify_type, cqg_manual_fill);
               }
            }

            EnableButtons();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "cel_ManualFillChanged", ex);
         }
      }

      /// <summary>
      /// This event is fired when the manual fill request is resolved or when some error has occurred during manual fills processing.
      /// </summary>
      /// <param name="cqg_manual_fill_request">Specifies the corresponding CQGManualFillRequest object.</param>
      /// <param name="cqg_error">Specifies the CQGError object describing the last error occurred during the processing of manual fills request or 
      /// Nothing/Invalid_Error in case of no error.
      /// CQGCEL.IsValid(Invalid_Error) returns False.</param>
      /// <remarks>
      /// If this event is a response to an addition request, the returned request object is already modified to include the TradeId of the added manual object.
      /// When the requested manual fill is executed, it will be announced through the CQGCEL.ManualFillChanged.
      /// </remarks>
      private void cel_ManualFillUpdateResolved(CQGManualFillRequest cqg_manual_fill_request, CQGError cqg_error)
      {
         if (m_CEL.IsValid(cqg_error))
         {
            ErrorHandler.ShowErrorMessage(string.Format("Error received on CQGCEL ManualFillUpdateResolved: {0}", cqg_error.Description), "Error", this);
         }
      }

      #endregion

      #region Auxiliary methods

      /// <summary>
      /// Initializes orders list view control and tab control used to 
      /// filter orders by their status.
      /// </summary>
      private void InitializeOrdersList()
      {
         // Configure list view
         lsvOrders.View = View.Details;
         lsvOrders.FullRowSelect = true;
         lsvOrders.GridLines = true;
         lsvOrders.Scrollable = true;

         // First column contains the original order ID (this column represents the item of the list)
         lsvOrders.Columns.Add("OrderID", 70, HorizontalAlignment.Left);

         // Add columns to list (this columns represent the sub-items of the list item)
         Type enumType = typeof(eOrdersListColumns);
         Array enumValues = Enum.GetValues(enumType);
         foreach (eOrdersListColumns val in enumValues)
         {
            lsvOrders.Columns.Add(val.ToString(), 70, HorizontalAlignment.Center);
         }

         // Add tabs for each order status
         TabPage tab;
         enumType = typeof(eOrderStatusTabs);
         enumValues = Enum.GetValues(enumType);
         foreach (eOrderStatusTabs val in enumValues)
         {
            tab = new TabPage(val.ToString());
            tab.Tag = val;
            tabControlOrders.TabPages.Add(tab);
         }

         tabControlOrders.SelectedIndex = tabControlOrders.TabPages.Count - 1;
      }

      /// <summary>
      /// Initializes manual fills list view control.
      /// </summary>
      private void InitializeManualFillsList()
      {
         // Configure list view
         lsvManualFills.View = View.Details;
         lsvManualFills.FullRowSelect = true;
         lsvManualFills.GridLines = true;
         lsvManualFills.Scrollable = true;

         // First column contains the manual fill ID (this column represents the item of the list)
         lsvManualFills.Columns.Add("TradeID", 70, HorizontalAlignment.Left);

         // Add columns to list (this columns represent the sub-items of the list item)
         Type enumType = typeof(eManualFillsListColumns);
         Array enumValues = Enum.GetValues(enumType);
         foreach (eManualFillsListColumns val in enumValues)
         {
            lsvManualFills.Columns.Add(val.ToString(), 70, HorizontalAlignment.Center);
         }

         // Add tabs for each order status
         TabPage tab = new TabPage(MANUAL_FILLS);
         tab.Tag = MANUAL_FILLS;
         tabControlOrders.TabPages.Add(tab);
         tabControlOrders.SelectedIndex = tabControlOrders.TabPages.Count - 2;
      }

      /// <summary>
      /// Configures and starts CQGCEL.
      /// </summary>
      private void InitializeCQGCEL()
      {
         m_CEL = new CQGCEL();

         // Configure CQG API. Based on this configuration CQG API works differently.
         m_CEL.APIConfiguration.CollectionsThrowException = false;
         m_CEL.APIConfiguration.DefaultInstrumentSubscriptionLevel = eDataSubscriptionLevel.dsQuotesAndBBA;
         m_CEL.APIConfiguration.ReadyStatusCheck = eReadyStatusCheck.rscOff;
         m_CEL.APIConfiguration.TimeZoneCode = eTimeZone.tzCentral;

         // Handle following events
         m_CEL.DataConnectionStatusChanged +=
             new _ICQGCELEvents_DataConnectionStatusChangedEventHandler(cel_DataConnectionStatusChanged);
         m_CEL.GWConnectionStatusChanged +=
             new _ICQGCELEvents_GWConnectionStatusChangedEventHandler(cel_GWConnectionStatusChanged);
         m_CEL.LineTimeChanged +=
             new _ICQGCELEvents_LineTimeChangedEventHandler(cel_LineTimeChanged);
         m_CEL.DataError +=
             new _ICQGCELEvents_DataErrorEventHandler(cel_DataError);
         m_CEL.AccountChanged +=
             new _ICQGCELEvents_AccountChangedEventHandler(cel_AccountChanged);
         m_CEL.InstrumentResolved += cel_InstrumentResolved;
         m_CEL.InstrumentChanged +=
             new _ICQGCELEvents_InstrumentChangedEventHandler(cel_InstrumentChanged);
         m_CEL.OrderChanged +=
             new _ICQGCELEvents_OrderChangedEventHandler(cel_OrderChanged);
         m_CEL.ManualFillsResolved +=
             new _ICQGCELEvents_ManualFillsResolvedEventHandler(cel_ManualFillsResolved);
         m_CEL.ManualFillChanged +=
             new _ICQGCELEvents_ManualFillChangedEventHandler(cel_ManualFillChanged);
         m_CEL.ManualFillUpdateResolved +=
             new _ICQGCELEvents_ManualFillUpdateResolvedEventHandler(cel_ManualFillUpdateResolved);

         // Start CQGCEL
         m_CEL.Startup();
      }

      /// <summary>
      /// Initializes orders' types combo control.
      /// </summary>
      private void InitializeOrderTypes()
      {
         cmbOrderTypes.Items.Add(eOrderType.otMarket);
         cmbOrderTypes.Items.Add(eOrderType.otLimit);
         cmbOrderTypes.Items.Add(eOrderType.otStop);
         cmbOrderTypes.Items.Add(eOrderType.otStopLimit);
         cmbOrderTypes.SelectedIndex = 0;
      }

      /// <summary>
      /// Initializes orders' open/close instructions combo control.
      /// </summary>
      private void InitializeOpenCloseInstructions()
      {
         cmbOpenCloseInstruction.Items.Add(eOpenCloseInstruction.ociUndefined);
         cmbOpenCloseInstruction.Items.Add(eOpenCloseInstruction.ociOpen);
         cmbOpenCloseInstruction.Items.Add(eOpenCloseInstruction.ociClose);
         cmbOpenCloseInstruction.Items.Add(eOpenCloseInstruction.ociClosePrevious);
         cmbOpenCloseInstruction.SelectedIndex = 0;
      }

      /// <summary>
      /// Initializes controls used to modify order's duration.
      /// </summary>
      private void InitializeDurationTypes()
      {
         cmbDuration.Items.Add(eOrderDuration.odDay);
         cmbDuration.Items.Add(eOrderDuration.odFAK);
         cmbDuration.Items.Add(eOrderDuration.odFOK);
         cmbDuration.Items.Add(eOrderDuration.odGoodTillCanceled);
         cmbDuration.Items.Add(eOrderDuration.odGoodTillDate);
         cmbDuration.Items.Add(eOrderDuration.odATC);
         cmbDuration.Items.Add(eOrderDuration.odATO);
         cmbDuration.Items.Add(eOrderDuration.odGoodTillTime);
         cmbDuration.Items.Add(eOrderDuration.odDayGoodTillTime);
         cmbDuration.SelectedIndex = 0;

         dtpOrderGoodTill.Format = DateTimePickerFormat.Custom;
         dtpOrderGoodTill.CustomFormat = DATE_FORMAT_SHORT;
      }

      /// <summary>
      /// Fills accounts list control with reloaded accounts.
      /// </summary>
      private void FillAccountsList()
      {
         cmbAccounts.Items.Clear();

         // Fill accounts to combo and hashtable
         string descriptiveName;
         foreach (CQGAccount acc in m_CEL.Accounts)
         {
            descriptiveName = string.Format("{0} ({1})", acc.GWAccountName, acc.GWAccountID);
            cmbAccounts.Items.Add(descriptiveName);
            m_AccountsMap[descriptiveName] = acc.GWAccountID;
         }

         // Select the first one
         if (cmbAccounts.Items.Count > 0)
         {
            cmbAccounts.SelectedIndex = 0;
         }
      }

      /// <summary>
      /// Clears all data.
      /// </summary>
      private void ClearInstrumentData()
      {
         string N_A = Resources.Resource.N_A;

         lblInstrumentDescription.Text = N_A;

         // Clear quotes
         lblAskPrice.Text = N_A;
         lblAskVolume.Text = N_A;
         lblBidPrice.Text = N_A;
         lblBidVolume.Text = N_A;
         lblTradePrice.Text = N_A;
         lblTradeVolume.Text = N_A;
      }

      /// <summary>
      /// Clears orders from the list.
      /// </summary>
      private void ClearOrders()
      {
         lsvOrders.Items.Clear();
      }

      /// <summary>
      /// Clears manual fills from the list.
      /// </summary>
      private void ClearManualFills()
      {
         lsvManualFills.Items.Clear();
      }

      /// <summary>
      /// Resolves the symbol and subscribes to the instrument.
      /// </summary>
      private void SubscribeInstrument()
      {
         try
         {
            // Send subscription request
            m_CEL.NewInstrument(txtSymbolName.Text);

            // Clear specified commodity
            txtSymbolName.Clear();
         }
         catch (Exception ex)
         {
            ErrorHandler.ShowErrorMessage(ex.Message, "Instrument subscription", this);
         }
      }

      /// <summary>
      /// Displays currently selected instrument data.
      /// </summary>
      /// <param name="Instrument">CQGInstrument object.</param>
      private void DisplayInstrumentProperties(CQGInstrument instrument)
      {
         lblInstrumentDescription.Text = instrument.Description;

         // Dump instrument quotes
         DisplayQuote(instrument, eQuoteType.qtAsk, lblAskPrice, lblAskVolume);
         DisplayQuote(instrument, eQuoteType.qtBid, lblBidPrice, lblBidVolume);
         DisplayQuote(instrument, eQuoteType.qtTrade, lblTradePrice, lblTradeVolume);
      }

      /// <summary>
      /// Displays instrument quote if valid. Otherwise N/A.
      /// </summary>
      /// <param name="instr">Instrument which quote is displayed</param>
      /// <param name="quoteType">Quote to display.</param>
      /// <param name="priceLabel">Quote price label.</param>
      /// <param name="volumeLabel">Quote volume label.</param>
      private void DisplayQuote(CQGInstrument instrument,
                                eQuoteType quoteType,
                                Label priceLabel,
                                Label volumeLabel)
      {
         CQGQuote quote = instrument.Quotes[quoteType];

         if (quote != null && quote.IsValid)
         {
            priceLabel.Text = instrument.ToDisplayPrice(quote.Price);
            volumeLabel.Text = quote.HasVolume ? quote.VolumeFractional.ToString() : Resources.Resource.N_A;
         }
         else
         {
            priceLabel.Text = Resources.Resource.N_A;
            volumeLabel.Text = Resources.Resource.N_A;
         }
      }

      /// <summary>
      /// Gets active instrument: the instrument, which is currently selected.
      /// </summary>
      /// <returns>Reference to active instrument instance.</returns>
      private CQGInstrument GetSelectedInstrument()
      {
         try
         {
            return lstInstruments.SelectedItem is null
               ? null
               : m_CEL.Instruments[lstInstruments.SelectedItem.ToString()];
         }
         catch
         {
            return null;
         }
      }

      /// <summary>
      /// Gets currently selected account.
      /// </summary>
      /// <returns>Reference to selected account instance.</returns>
      private CQGAccount GetSelectedAccount()
      {
         try
         {
            int gwAccountID = (int)m_AccountsMap[cmbAccounts.SelectedItem];
            return m_CEL.Accounts[gwAccountID];
         }
         catch
         {
            return null;
         }
      }

      /// <summary>
      /// Places order, based on application GUI state.
      /// </summary>
      /// <param name="orderSide">Order side.</param>
      private void PlaceOrder(eOrderSide orderSide)
      {
         if (orderSide == eOrderSide.osdUndefined)
         {
            throw new ArgumentException("Order side is not defined.");
         }

         CQGAccount account = GetSelectedAccount();
         if (account == null)
         {
            throw new NullReferenceException("No account is selected.");
         }

         CQGInstrument instrument = GetSelectedInstrument();
         if (instrument == null)
         {
            throw new NullReferenceException("No instrument is selected.");
         }

         eOrderType orderType = (eOrderType)(cmbOrderTypes.SelectedItem);

         var quantity = nudQuantity.Value;
         if (quantity == 0)
         {
            throw new ArgumentException("Order quantity should not be zero.");
         }
         // Order side can be specified in two ways: 
         //      * if UseOrderSide is set in APIConfig then we need to specify side 
         //        explicitly, and order quantity must be greater than 0.
         //      * if setting below is not set the side is detected by order quantity sign.
         //        Negative quantity specifies sell side.
         // Here we use the second one
         if (orderSide == eOrderSide.osdSell)
         {
            quantity *= -1;
         }

         if (m_IsManualFillMode)
         {
            AddManualFill(orderSide, account, instrument, quantity, orderType);
         }
         else
         {
            // Default values of prices
            double limitPrice = 0.0;
            double stopPrice = 0.0;

            var constructor = m_CEL.CreateOrderConstructor(orderType, eOrderSide.osdUndefined);
            constructor.Properties[eOrderProperty.opInstrumentNameDetailed].Value = instrument.DetailedName;
            constructor.Properties[eOrderProperty.opAccountID].Value = account.GWAccountID;
            constructor.Properties[eOrderProperty.opQuantityFractional].Value = quantity;
            constructor.Properties[eOrderProperty.opUEName].Value = string.Empty;

            if (orderType == eOrderType.otLimit || orderType == eOrderType.otStopLimit)
            {
               if (txtLimitPrice.Text == Resources.Resource.N_A)
               {
                  throw new ArgumentException("Limit price is not valid.");
               }
               limitPrice = instrument.FromDisplayPrice(txtLimitPrice.Text);
               constructor.Properties[eOrderProperty.opLimitPrice].Value = limitPrice;
            }

            if (orderType == eOrderType.otStop || orderType == eOrderType.otStopLimit)
            {
               if (txtStopPrice.Text == Resources.Resource.N_A)
               {
                  throw new ArgumentException("Stop price is not valid.");
               }
               stopPrice = instrument.FromDisplayPrice(txtStopPrice.Text);
               constructor.Properties[eOrderProperty.opStopPrice].Value = stopPrice;
            }

            // Create order, since we have already all the needed parameters
            CQGOrder order = constructor.CreateOrder();

            // Set order parked status
            if (chkParked.Checked)
            {
               order.Properties[eOrderProperty.opParked].Value = true;
            }

            // Set order duration
            eOrderDuration durationType = (eOrderDuration)cmbDuration.SelectedItem;
            order.Properties[eOrderProperty.opDurationType].Value = durationType;
            if (durationType == eOrderDuration.odGoodTillDate)
            {
               // You should always pass UTC time to API calls
               order.Properties[eOrderProperty.opGTDTime].Value = TimeZoneInfo.ConvertTimeToUtc(dtpOrderGoodTill.Value);
            }
            else if (durationType == eOrderDuration.odGoodTillTime || durationType == eOrderDuration.odDayGoodTillTime)
            {
               // You should always pass UTC time to API calls
               order.Properties[eOrderProperty.opGTTTime].Value = TimeZoneInfo.ConvertTimeToUtc(dtpOrderGoodTill.Value);
            }

            order.Properties[eOrderProperty.opOpenCloseInstruction].Value = (eOpenCloseInstruction)cmbOpenCloseInstruction.SelectedItem;

            if (chkBracket.Checked)
            {
               // Open bracket properties
               frmBracketProperties brktProps = new frmBracketProperties(m_CEL, order, orderSide);
               brktProps.ShowDialog(this);
            }
            else
            {
               // Place it
               order.Place();
            }
         }
      }

      /// <summary>
      /// Displays currently selected account's orders.
      /// </summary>
      private void DisplayOrders()
      {
         // Clear old data
         ClearOrders();

         // Get current selected account
         CQGAccount account = GetSelectedAccount();
         if (account != null)
         {
            foreach (CQGOrder order in account.InternalOrders)
            {
               DisplayOrder(eChangeType.ctAdded, order);
            }
         }
      }

      /// <summary>
      /// Reflects order change in the list.
      /// </summary>
      /// <param name="change">Order change type.</param>
      /// <param name="order">CQGOrder object.</param>
      private void DisplayOrder(eChangeType change, CQGOrder order)
      {
         // Find/add order row in the list
         int index;
         if (change == eChangeType.ctAdded)
         {
            index = lsvOrders.Items.Count;
         }
         else
         {
            ListViewItem foundItem = null;
            // Find item in the list
            foreach (ListViewItem item in lsvOrders.Items)
            {
               if (item.Text == order.OriginalOrderID)
               {
                  foundItem = item;
                  break;
               }
            }

            if (foundItem == null)
            {
               // Order may be absent in the list because of its previous status
               index = lsvOrders.Items.Count;
            }
            else
            {
               index = foundItem.Index;
               lsvOrders.Items.Remove(foundItem);
            }
         }

         // Get selected tab
         eOrderStatusTabs selectedList = (eOrderStatusTabs)tabControlOrders.SelectedTab.Tag;

         // Add order if its status satisfies selected tab
         if (change != eChangeType.ctRemoved &&
             (selectedList & GetWhereToAddOrder(order)) == selectedList)
         {
            ListViewItem item = lsvOrders.Items.Insert(index, order.OriginalOrderID);

            // Add order as a tag
            item.Tag = order;

            // Add subitems for each column
            Type enumType = typeof(eOrdersListColumns);
            Array enumValues = Enum.GetValues(enumType);
            for (int i = 0; i < enumValues.GetLength(0); i++)
            {
               item.SubItems.Add(string.Empty);
            }

            item.SubItems[(int)eOrdersListColumns.Side].Text = (order.QuantityFractional > 0) ? ORDER_SIDE_BUY : ORDER_SIDE_SELL;

            if (order.FilledQuantityFractional != order.QuantityFractional)
            {
               item.SubItems[(int)eOrdersListColumns.Quantity].Text = 
                  string.Format("{0} [{1}]", Math.Abs(order.QuantityFractional).ToString(), Math.Abs(order.FilledQuantityFractional).ToString());
            }
            else
            {
               item.SubItems[(int)eOrdersListColumns.Quantity].Text = Math.Abs(order.QuantityFractional).ToString();
            }

            item.SubItems[(int)eOrdersListColumns.Instrument].Text = order.InstrumentName;
            item.SubItems[(int)eOrdersListColumns.Type].Text = order.Type.ToString();
            item.SubItems[(int)eOrdersListColumns.Status].Text = order.GWStatus.ToString();
            item.SubItems[(int)eOrdersListColumns.PlaceTime].Text = order.PlaceTime.ToString(DATE_FORMAT_LONG);
            item.SubItems[(int)eOrdersListColumns.Duration].Text = order.DurationType.ToString();

            // Display good till date if necessary
            if (order.DurationType == eOrderDuration.odGoodTillDate)
            {
               DateTime GTDTime = (DateTime)order.Properties[eOrderProperty.opGTDTime].Value;

               item.SubItems[(int)eOrdersListColumns.GoodTill].Text = GTDTime.ToString(DATE_FORMAT_SHORT);

            }
            else if (order.DurationType == eOrderDuration.odGoodTillTime ||
               order.DurationType == eOrderDuration.odDayGoodTillTime)
            {
               DateTime GTDTime = (DateTime)order.Properties[eOrderProperty.opGTTTime].Value;
               item.SubItems[(int)eOrdersListColumns.GoodTill].Text = GTDTime.ToString(TIME_FORMAT);
            }
            else
            {
               item.SubItems[(int)eOrdersListColumns.GoodTill].Text = Resources.Resource.N_A;
            }

            // Display prices
            item.SubItems[(int)eOrdersListColumns.LimitPrice].Text =
                    AuxiliaryMethods.GetDisplayPrice(m_CEL, order.Instrument, order.LimitPrice);

            item.SubItems[(int)eOrdersListColumns.StopPrice].Text =
                    AuxiliaryMethods.GetDisplayPrice(m_CEL, order.Instrument, order.StopPrice);

            item.SubItems[(int)eOrdersListColumns.AvgFillPrice].Text =
                    AuxiliaryMethods.GetDisplayPrice(m_CEL, order.Instrument, (double)order.Properties[eOrderProperty.opAverageFillPrice].Value);
         }
      }

      /// <summary>
      /// Displays currently selected account's manual fills.
      /// </summary>
      private void DisplayManualFills()
      {
         // Clear old data
         ClearManualFills();

         // Get current selected account
         CQGAccount account = GetSelectedAccount();
         if (account != null)
         {
            CQGManualFills manualFills = m_CEL.AllManualFills[account.GWAccountID];

            if (manualFills == null)
            {
               return;
            }

            foreach (CQGManualFill manualFill in manualFills)
            {
               DisplayManualFill(eManualFillUpdateType.mfutAdd, manualFill);
            }
         }
      }

      /// <summary>
      /// Reflects manual fill change in the list.
      /// </summary>
      /// <param name="change">Manual fill update type.</param>
      /// <param name="order">CQGManualFill object.</param>
      private void DisplayManualFill(eManualFillUpdateType change, CQGManualFill manualFill)
      {
         // Find/add manual fill row in the list
         int index;
         if (change == eManualFillUpdateType.mfutAdd)
         {
            index = lsvManualFills.Items.Count;
         }
         else
         {
            ListViewItem foundItem = null;
            // Find item in the list
            foreach (ListViewItem item in lsvManualFills.Items)
            {
               if (item.Text == manualFill.TradeId.ToString())
               {
                  foundItem = item;
                  break;
               }
            }

            if (foundItem == null)
            {
               // Manual fill may be absent in the list because of its previous status
               index = lsvManualFills.Items.Count;
            }
            else
            {
               index = foundItem.Index;
               lsvManualFills.Items.Remove(foundItem);
            }
         }

         // Add manual fill if its status satisfies selected tab
         if (change != eManualFillUpdateType.mfutDelete && !IsOrdersTab())
         {
            ListViewItem item = lsvManualFills.Items.Insert(index, manualFill.TradeId.ToString());

            // Add manual fill as a tag
            item.Tag = manualFill;

            // Add subitems for each column
            Type enumType = typeof(eManualFillsListColumns);
            Array enumValues = Enum.GetValues(enumType);
            for (int i = 0; i < enumValues.GetLength(0); i++)
            {
               item.SubItems.Add(string.Empty);
            }

            item.SubItems[(int)eManualFillsListColumns.Instrument].Text = manualFill.InstrumentName;
            item.SubItems[(int)eManualFillsListColumns.Side].Text = (manualFill.QuantityFractional > 0) ? ORDER_SIDE_BUY : ORDER_SIDE_SELL;
            item.SubItems[(int)eManualFillsListColumns.Quantity].Text = Math.Abs(manualFill.QuantityFractional).ToString();
            item.SubItems[(int)eManualFillsListColumns.Price].Text = manualFill.DisplayPrice;
            item.SubItems[(int)eManualFillsListColumns.Currency].Text = getCurrencyByInstrumentFullName(manualFill.InstrumentName);
            item.SubItems[(int)eManualFillsListColumns.FillDateTime].Text = manualFill.TradeTimestamp.ToString(DATE_FORMAT_LONG);
            item.SubItems[(int)eManualFillsListColumns.Note].Text = manualFill.Note;
            item.SubItems[(int)eManualFillsListColumns.StatementDate].Text = manualFill.StatementDate.ToString(DATE_FORMAT_SHORT);
         }
      }

      /// <summary>
      /// Returns the currency abbreviation string from given instrument full name.
      /// </summary>
      /// <param name="curInstrumentFullName">Given instrument full name to get currency abbreviation.</param>
      /// <returns>Currency abbreviation string.</returns>
      private string getCurrencyByInstrumentFullName(string curInstrumentFullName)
      {
         CQGInstrument instrument = m_CEL.Instruments[curInstrumentFullName];
         return instrument != null ? instrument.Currency : string.Empty;
      }

      /// <summary>
      /// Gets the tabs where we need to add added/changed order.
      /// </summary>
      /// <param name="order">CQGOrder instance.</param>
      /// <returns>Tabs where we need to add order.</returns>
      private eOrderStatusTabs GetWhereToAddOrder(CQGOrder order)
      {
         // This tab shows all orders
         eOrderStatusTabs lists = eOrderStatusTabs.All;

         switch (order.GWStatus)
         {
            case eOrderStatus.osContingent:
            case eOrderStatus.osParked:
               lists |= eOrderStatusTabs.Parked;
               break;
            case eOrderStatus.osFilled:
               lists |= eOrderStatusTabs.Filled;
               break;
            case eOrderStatus.osInOrderBook:
            case eOrderStatus.osInTransit:
               lists |= eOrderStatusTabs.Working;
               break;
            case eOrderStatus.osCanceled:
               if (Math.Abs(order.FilledQuantityFractional) > 0)
               {
                  lists |= eOrderStatusTabs.Filled | eOrderStatusTabs.Canceled;
               }
               else
               {
                  lists |= eOrderStatusTabs.Canceled;
               }
               break;
            case eOrderStatus.osRejectFCM:
            case eOrderStatus.osRejectGW:
               lists |= eOrderStatusTabs.Exception;
               break;
         }

         return lists;
      }

      /// <summary>
      /// Sets specified instrument's current trade price to limit/stop prices controls, based on 
      /// specified order type. Also, sets precision and increment steps of controls.
      /// </summary>
      /// <param name="instrument">CQGInstrument object.</param>
      /// <param name="orderType">Order type.</param>
      private void SetOrderPrices(CQGInstrument instrument, eOrderType orderType)
      {
         txtLimitPrice.Text = Resources.Resource.N_A;
         txtLimitPrice.Enabled = false;
         vsbLimitPrice.Enabled = false;

         txtStopPrice.Text = Resources.Resource.N_A;
         txtStopPrice.Enabled = false;
         vsbStopPrice.Enabled = false;

         if (orderType == eOrderType.otLimit || orderType == eOrderType.otStopLimit)
         {
            txtLimitPrice.Text = AuxiliaryMethods.GetValidPrice(instrument);
            txtLimitPrice.Enabled = true;
            vsbLimitPrice.Enabled = true;
         }

         if (orderType == eOrderType.otStop || orderType == eOrderType.otStopLimit)
         {
            txtStopPrice.Text = AuxiliaryMethods.GetValidPrice(instrument);
            txtStopPrice.Enabled = true;
            vsbStopPrice.Enabled = true;
         }
      }

      /// <summary>
      /// Enables/disables controls connected with Data server.
      /// </summary>
      /// <param name="enable">Controls enablement status.</param>
      private void EnableDataControls(bool enable)
      {
         grbInstruments.Enabled = enable;
      }

      /// <summary>
      /// Enables/disables controls connected with Gateway server
      /// </summary>
      /// <param name="enable">Controls enablement status.</param>
      private void EnableGWControls(bool enable)
      {
         grbAccounts.Enabled = enable;
         grbOrderProperties.Enabled = enable;
         btnBuy.Enabled = enable;
         btnSell.Enabled = enable;
      }

      /// <summary>
      /// Checks whether the curent tab is orders tab or not.
      /// </summary>
      /// <returns>True, if the selected tab is orders tab; otherwise - False.</returns>
      private bool IsOrdersTab()
      {
         return tabControlOrders.SelectedTab.Text != MANUAL_FILLS;
      }

      /// <summary>
      /// Enables/disables Modify, Activate and Cancel button
      /// </summary>
      private void EnableButtons()
      {
         bool isOrdersTab = IsOrdersTab();
         bool hasOrders = lsvOrders.Items.Count > 0;
         bool hasSelectedOrders = lsvOrders.SelectedItems.Count > 0;
         bool hasSelectedManualFills = lsvManualFills.SelectedItems.Count > 0;
         bool hasMAnualFills = lsvManualFills.Items.Count > 0;

         btnModify.Enabled = isOrdersTab ? hasSelectedOrders : hasSelectedManualFills;
         btnModify.Visible = !isOrdersTab ||
                             tabControlOrders.SelectedTab.Text == eOrderStatusTabs.Working.ToString() ||
                             tabControlOrders.SelectedTab.Text == eOrderStatusTabs.Parked.ToString();

         btnCancel.Enabled = btnModify.Enabled;
         btnCancel.Text = isOrdersTab ? "Cancel" : "Delete";
         btnCancel.Visible = btnModify.Visible;

         btnCancelAll.Enabled = isOrdersTab ? hasOrders : hasMAnualFills;
         btnCancelAll.Text = isOrdersTab ? "Cancel All" : "Delete All";
         btnCancelAll.Visible = btnCancel.Visible;

         btnActivate.Enabled = isOrdersTab && hasSelectedOrders;
         btnActivate.Visible = isOrdersTab && tabControlOrders.SelectedTab.Text == eOrderStatusTabs.Parked.ToString();
         btnActivateAll.Enabled = isOrdersTab && hasOrders;
         btnActivateAll.Visible = btnActivate.Visible;
      }

      /// <summary>
      /// Returns string representation of the connection status.
      /// </summary>
      /// <param name="status">Connection status.</param>
      /// <returns>String representation of the connection status</returns>
      private string ConnectionStatusToString(eConnectionStatus status)
      {
         string connectionStatus;

         switch (status)
         {
            case eConnectionStatus.csConnectionUp:
               connectionStatus = "Up";
               break;
            case eConnectionStatus.csConnectionDelayed:
               connectionStatus = "Delayed";
               break;
            case eConnectionStatus.csConnectionDown:
               connectionStatus = "Down";
               break;
            case eConnectionStatus.csConnectionTrouble:
               connectionStatus = "Trouble";
               break;
            case eConnectionStatus.csConnectionNotLoggedOn:
               connectionStatus = "Not logged on";
               break;
            default:
               throw new ArgumentException("Unknown connection status.", "newStatus");
         }

         return connectionStatus;
      }

      /// <summary>
      /// Adds a new manual fill.
      /// </summary>
      private void AddManualFill(eOrderSide side, CQGAccount account, CQGInstrument instrument, decimal quantity, eOrderType type)
      {
         try
         {
            decimal fillPrice = decimal.Zero;
            decimal.TryParse(AuxiliaryMethods.GetValidPrice(instrument), out fillPrice);
            int priceIncrement = 1;

            int.TryParse(instrument.ToDisplayPrice(instrument.GetTickSizeFor(instrument.Trade.Price)), out priceIncrement);

            dlgManualFill tmpManualFillDlg = new dlgManualFill();

            tmpManualFillDlg.Text = string.Format("Add Manual Fill");
            tmpManualFillDlg.cmbManualFillAction.SelectedIndex = side == eOrderSide.osdSell ? 1 : 0;
            tmpManualFillDlg.nudManualFillSize.Value = Math.Abs(quantity);
            tmpManualFillDlg.nudFillPrice.Increment = priceIncrement;
            tmpManualFillDlg.nudFillPrice.Value = fillPrice;
            tmpManualFillDlg.dtpManualFillTime.Value = DateTime.Now;
            tmpManualFillDlg.dtpStatementDate.Value = DateTime.Now;

            if (tmpManualFillDlg.ShowDialog(this) == DialogResult.OK)
            {
               CQGManualFillRequest tmpManualFillRequest = m_CEL.CreateManualFillRequest(eManualFillUpdateType.mfutAdd);

               tmpManualFillRequest.GWAccountID = account.GWAccountID;
               tmpManualFillRequest.InstrumentName = instrument.FullName;
               tmpManualFillRequest.Note = tmpManualFillDlg.txtManualFillNote.Text;
               tmpManualFillRequest.Price = instrument.FromDisplayPrice(tmpManualFillDlg.nudFillPrice.Value.ToString());
               tmpManualFillRequest.QuantityFractional = tmpManualFillDlg.nudManualFillSize.Value * (tmpManualFillDlg.cmbManualFillAction.SelectedIndex > 0 ? -1 : 1);
               tmpManualFillRequest.Side = eOrderSide.osdUndefined;
               tmpManualFillRequest.TradeTimestamp = tmpManualFillDlg.dtpManualFillTime.Value;
               tmpManualFillRequest.StatementDate = tmpManualFillDlg.dtpStatementDate.Value;

               m_CEL.RequestManualFillUpdate(tmpManualFillRequest);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "AddManualFill", ex);
         }
      }

      /// <summary>
      /// Modifies the existing manual fill.
      /// </summary>
      /// <param name="manualFill">CQGManualFill object to modify.</param>
      private void ModifyManualFill(CQGManualFill manualFill)
      {
         try
         {
            if (manualFill == null)
            {
               return;
            }

            int priceIncrement = 1;
            CQGInstrument instrument = GetSelectedInstrument();
            if (instrument != null)
            {
               int.TryParse(instrument.ToDisplayPrice(instrument.GetTickSizeFor(instrument.Trade.Price)), out priceIncrement);
            }

            dlgManualFill tmpManualFillDlg = new dlgManualFill();

            tmpManualFillDlg.Text = string.Format("Modify Manual Fill");
            tmpManualFillDlg.cmbManualFillAction.SelectedIndex = manualFill.QuantityFractional < 0 ? 1 : 0;
            tmpManualFillDlg.dtpManualFillTime.Value = manualFill.TradeTimestamp;
            tmpManualFillDlg.nudManualFillSize.Value = Math.Abs(manualFill.QuantityFractional);
            tmpManualFillDlg.dtpStatementDate.Value = manualFill.StatementDate;
            tmpManualFillDlg.txtManualFillNote.Text = manualFill.Note;
            tmpManualFillDlg.nudFillPrice.Increment = priceIncrement;
            tmpManualFillDlg.nudFillPrice.Value = decimal.Parse(instrument.ToDisplayPrice(manualFill.Price));

            if (tmpManualFillDlg.ShowDialog(this) == DialogResult.OK)
            {
               CQGManualFillRequest tmpManualFillRequest = m_CEL.CreateManualFillRequest(eManualFillUpdateType.mfutModify, manualFill);

               tmpManualFillRequest.Note = tmpManualFillDlg.txtManualFillNote.Text;
               tmpManualFillRequest.Price = instrument.FromDisplayPrice(tmpManualFillDlg.nudFillPrice.Value.ToString());
               tmpManualFillRequest.QuantityFractional = tmpManualFillDlg.nudManualFillSize.Value * (tmpManualFillDlg.cmbManualFillAction.SelectedIndex > 0 ? -1 : 1);
               tmpManualFillRequest.Side = eOrderSide.osdUndefined;
               tmpManualFillRequest.TradeTimestamp = tmpManualFillDlg.dtpManualFillTime.Value;
               tmpManualFillRequest.StatementDate = tmpManualFillDlg.dtpStatementDate.Value;

               m_CEL.RequestManualFillUpdate(tmpManualFillRequest);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "ModifyManualFill", ex);
         }
      }

      /// <summary>
      /// Deletes the existing manual fill.
      /// </summary>
      /// <param name="manualFill">CQGManualFill objects to delete.</param>
      private void DeleteManualFill(CQGManualFill manualFill)
      {
         try
         {
            if (manualFill == null)
            {
               return;
            }

            CQGManualFillRequest tmpManualFillRequest = m_CEL.CreateManualFillRequest(eManualFillUpdateType.mfutDelete, manualFill);

            m_CEL.RequestManualFillUpdate(tmpManualFillRequest);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "DeleteManualFill", ex);
         }
      }

      #endregion

      #region Static methods

      /// <summary>
      /// The main entry point for the application.
      /// </summary>
      [STAThread]
      static void Main()
      {
         try
         {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new frmOrderPlacing());
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "Main", ex);
         }
      }

      #endregion

      #region Form controls event handlers

      /// <summary>
      /// Initializes CQGCEL object when form is loaded.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void frmOrderPlacing_Load(object sender, EventArgs e)
      {
         try
         {
            InitializeCQGCEL();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "frmOrderPlacing_Load", ex);
         }
      }

      /// <summary>
      /// Subscribe to the new instrument when the button is clicked.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void btnSubscribe_Click(object sender, EventArgs e)
      {
         SubscribeInstrument();
      }

      /// <summary>
      /// Used to display newly selected instrument data.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void lstInstruments_SelectedIndexChanged(object sender, EventArgs e)
      {
         try
         {
            CQGInstrument instrument = GetSelectedInstrument();
            if (instrument == null)
            {
               ClearInstrumentData();
            }
            else
            {
               DisplayInstrumentProperties(instrument);
            }

            // Set new prices for selected instrument
            SetOrderPrices(instrument, (eOrderType)cmbOrderTypes.SelectedItem);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "lstInstruments_SelectedIndexChanged", ex);
         }
      }

      /// <summary>
      /// Buys order.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void btnBuy_Click(object sender, EventArgs e)
      {
         try
         {
            //nudQuantity.Value
            PlaceOrder(eOrderSide.osdBuy);
         }
         catch (Exception ex)
         {
            ErrorHandler.ShowErrorMessage(ex.Message, "Order placing", this);
         }
      }

      /// <summary>
      /// Sells order.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void btnSell_Click(object sender, EventArgs e)
      {
         try
         {
            PlaceOrder(eOrderSide.osdSell);
         }
         catch (Exception ex)
         {
            ErrorHandler.ShowErrorMessage(ex.Message, "Order placing", this);
         }
      }

      /// <summary>
      /// Cancels selected orders.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void btnCancel_Click(object sender, EventArgs e)
      {
         try
         {
            if (IsOrdersTab())
            {
               DialogResult dlgRes = MessageBox.Show(this,
                   "Are you sure you want to cancel selected order(s)?",
                   "Selected order(s) cancelation confirmation",
                   MessageBoxButtons.YesNo,
                   MessageBoxIcon.Question);

               if (dlgRes == DialogResult.Yes)
               {
                  CQGOrder order;
                  foreach (ListViewItem item in lsvOrders.SelectedItems)
                  {
                     order = item.Tag as CQGOrder;
                     if (order != null)
                     {
                        order.Cancel();
                     }
                  }
               }
            }
            else
            {
               DialogResult dlgRes = MessageBox.Show(this,
                   "Are you sure you want to delete selected manual fill(s)?",
                   "Selected manual fill(s) deletion confirmation",
                   MessageBoxButtons.YesNo,
                   MessageBoxIcon.Question);

               if (dlgRes == DialogResult.Yes)
               {
                  CQGManualFill manualFill;
                  foreach (ListViewItem item in lsvManualFills.SelectedItems)
                  {
                     manualFill = item.Tag as CQGManualFill;

                     DeleteManualFill(manualFill);
                  }
               }
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "btnCancel_Click", ex);
         }
      }

      /// <summary>
      /// Activates selected parked orders.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void btnActivate_Click(object sender, EventArgs e)
      {
         try
         {
            DialogResult dlgRes = MessageBox.Show(this,
                "Are you sure you want to activate selected parked order(s)?",
                "Selected parked order(s) activation confirmation",
                MessageBoxButtons.YesNo,
                MessageBoxIcon.Question);

            if (dlgRes == DialogResult.Yes)
            {
               CQGOrder order;
               foreach (ListViewItem item in lsvOrders.SelectedItems)
               {
                  order = item.Tag as CQGOrder;
                  if (order != null)
                  {
                     order.Activate();
                  }
               }
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "btnActivate_Click", ex);
         }
      }

      /// <summary>
      /// Cancels selected account's all orders.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void btnCancelAll_Click(object sender, EventArgs e)
      {
         try
         {
            if (IsOrdersTab())
            {
               CQGAccount account = GetSelectedAccount();
               if (account == null)
               {
                  return;
               }

               if (account.Orders.Count == 0)
               {
                  return;
               }

               DialogResult dlgRes = MessageBox.Show(this,
                   "Are you sure you want to cancel all orders?",
                   "Account orders cancelation confirmation",
                   MessageBoxButtons.YesNo,
                   MessageBoxIcon.Question);

               if (dlgRes == DialogResult.Yes)
               {
                  if (tabControlOrders.SelectedTab.Text == eOrderStatusTabs.Parked.ToString())
                  {
                     m_CEL.CancelAllOrders(account, null, false, true, eOrderSide.osdUndefined);
                  }
                  else
                  {
                     m_CEL.CancelAllOrders(account, null, false, false, eOrderSide.osdUndefined);
                  }
               }
            }
            else
            {
               DialogResult dlgRes = MessageBox.Show(this,
                   "Are you sure you want to delete all manual fills?",
                   "Account manual fills deletion confirmation",
                   MessageBoxButtons.YesNo,
                   MessageBoxIcon.Question);

               if (dlgRes == DialogResult.Yes)
               {
                  CQGManualFill manualFill;
                  foreach (ListViewItem item in lsvManualFills.Items)
                  {
                     manualFill = item.Tag as CQGManualFill;

                     DeleteManualFill(manualFill);
                  }
               }
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "btnCancelAll_Click", ex);
         }
      }

      /// <summary>
      /// Activates selected account's all parked orders.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void btnActivateAll_Click(object sender, EventArgs e)
      {
         try
         {
            CQGAccount account = GetSelectedAccount();
            if (account == null)
            {
               return;
            }

            if (account.Orders.Count == 0)
            {
               return;
            }

            DialogResult dlgRes = MessageBox.Show(this,
                "Are you sure you want to activate all parked orders?",
                "Account parked orders activation confirmation",
                MessageBoxButtons.YesNo,
                MessageBoxIcon.Question);

            if (dlgRes == DialogResult.Yes)
            {
               m_CEL.ActivateAllOrders(account, null, false, eOrderSide.osdUndefined);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "btnActivateAll_Click", ex);
         }
      }

      /// <summary>
      /// Increments/Decrements limit price.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A ScrollEventArgs that contains the event data. </param>
      private void vsbLimitPrice_Scroll(object sender, ScrollEventArgs e)
      {
         try
         {
            AuxiliaryMethods.UpDownPrice(GetSelectedInstrument(), txtLimitPrice, e);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "vsbLimitPrice_Scroll", ex);
         }
      }

      /// <summary>
      /// Increments/Decrements stop price.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A ScrollEventArgs that contains the event data. </param>
      private void vsbStopPrice_Scroll(object sender, ScrollEventArgs e)
      {
         try
         {
            AuxiliaryMethods.UpDownPrice(GetSelectedInstrument(), txtStopPrice, e);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "vsbStopPrice_Scroll", ex);
         }
      }

      /// <summary>
      /// Subscribe to the new instrument when the "Enter" button is clicked.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void txtSymbolName_KeyDown(object sender, KeyEventArgs e)
      {
         if (e.KeyCode == Keys.Enter)
         {
            SubscribeInstrument();
         }
      }

      /// <summary>
      /// Removes selected instrument from the list and CQGCEL.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void btnRemoveInstrument_Click(object sender, EventArgs e)
      {
         try
         {
            CQGInstrument selInstrument = GetSelectedInstrument();
            if (selInstrument == null)
            {
               return;
            }

            // Remove from list
            lstInstruments.Items.Remove(selInstrument.FullName);
            if (lstInstruments.Items.Count > 0)
            {
               // Select first in the list
               lstInstruments.SelectedIndex = 0;
            }
            else
            {
               // Nothing to select, clear old data
               ClearInstrumentData();
            }

            // Remove instrument from CEL
            m_CEL.RemoveInstrument(selInstrument);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "btnRemoveInstrument_Click", ex);
         }
      }

      /// <summary>
      /// Provides faster method to set quantity.
      /// </summary>
      /// <param name="sender">The source of the event. 
      /// Used to determine which quantity button was clicked.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void btnQuantities_CheckedChanged(object sender, EventArgs e)
      {
         try
         {
            Button quantityButton = sender as Button;
            if (quantityButton == null)
            {
               throw new ArgumentException("Unexpected event sender type. Sender must be a Button.", "sender");
            }

            nudQuantity.Value = Decimal.Parse(quantityButton.Text);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "btnQuantities_CheckedChanged", ex);
         }
      }

      /// <summary>
      /// Opens windows to specify properties and modify order.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void btnModify_Click(object sender, EventArgs e)
      {
         try
         {
            if (IsOrdersTab())
            {
               if (lsvOrders.SelectedItems.Count == 1)
               {
                  CQGOrder order = lsvOrders.SelectedItems[0].Tag as CQGOrder;
                  if (order.CanBeModified)
                  {
                     frmModify modForm = new frmModify(m_CEL, order);
                     modForm.ShowDialog(this);
                  }
                  else
                  {
                     MessageBox.Show(this,
                         "This order can't be modified.",
                         "Modification message",
                         MessageBoxButtons.OK,
                         MessageBoxIcon.Exclamation);
                  }
               }
               else
               {
                  MessageBox.Show(this,
                      "Please select one order to modify.",
                      "Modification message",
                      MessageBoxButtons.OK,
                      MessageBoxIcon.Exclamation);
               }
            }
            else
            {
               if (lsvManualFills.SelectedItems.Count == 1)
               {
                  CQGManualFill manualFill = lsvManualFills.SelectedItems[0].Tag as CQGManualFill;

                  ModifyManualFill(manualFill);
               }
               else
               {
                  MessageBox.Show(this,
                      "Please select one manual fill to modify.",
                      "Modification message",
                      MessageBoxButtons.OK,
                      MessageBoxIcon.Exclamation);
               }
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "btnModify_Click", ex);
         }
      }

      /// <summary>
      /// Filters and shows orders by their status based on the selected tab.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void tabControlOrders_SelectedIndexChanged(object sender, EventArgs e)
      {
         try
         {
            if (IsOrdersTab())
            {
               lsvOrders.Visible = true;
               lsvManualFills.Visible = false;

               DisplayOrders();
            }
            else
            {
               lsvOrders.Visible = false;
               lsvManualFills.Visible = true;

               DisplayManualFills();
            }

            EnableButtons();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "tabControlOrders_SelectedIndexChanged", ex);
         }
      }

      /// <summary>
      /// Shows newly selected account's orders.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void cmbAccounts_SelectedIndexChanged(object sender, EventArgs e)
      {
         try
         {
            if (IsOrdersTab())
            {
               DisplayOrders();
            }
            else
            {
               DisplayManualFills();
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "cmbAccounts_SelectedIndexChanged", ex);
         }
      }

      /// <summary>
      /// Recalculate prices when order type is changed.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data.</param>
      private void cmbOrderTypes_SelectedIndexChanged(object sender, EventArgs e)
      {
         try
         {
            SetOrderPrices(GetSelectedInstrument(), (eOrderType)cmbOrderTypes.SelectedItem);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "cmbOrderTypes_SelectedIndexChanged", ex);
         }
      }

      /// <summary>
      /// Enables/disables GTD control based on selected duration type.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data.</param>
      private void cmbDuration_SelectedIndexChanged(object sender, EventArgs e)
      {
         try
         {
            eOrderDuration orderDuration = (eOrderDuration)cmbDuration.SelectedItem;
            dtpOrderGoodTill.Enabled = orderDuration == eOrderDuration.odGoodTillDate || 
               orderDuration == eOrderDuration.odGoodTillTime || orderDuration == eOrderDuration.odDayGoodTillTime;
            if (orderDuration == eOrderDuration.odGoodTillDate)
            {
               dtpOrderGoodTill.CustomFormat = DATE_FORMAT_SHORT;
            }
            else if (orderDuration == eOrderDuration.odGoodTillTime || orderDuration == eOrderDuration.odDayGoodTillTime)
            {
               dtpOrderGoodTill.CustomFormat = DATE_FORMAT_LONG;
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmOrderPlacing", "cmbDuration_SelectedIndexChanged", ex);
         }
      }

      /// <summary>
      /// Shutdowns CQGCEL before closing the form.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void frmOrderPlacing_Closing(object sender, System.ComponentModel.CancelEventArgs e)
      {
         try
         {
            m_CEL.Shutdown();
         }
         catch
         { }
      }

      /// <summary>
      /// Open CQG API web page.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A LinkLabelLinkClickedEventArgs that contains the event data.</param>
      private void llWeb_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
      {
         System.Diagnostics.Process.Start("iexplore.exe", "http://www.cqg.com/Products/CQG-API.aspx");
      }

      /// <summary>
      /// Enable/disable buttons.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void lsvOrders_SelectedIndexChanged(object sender, EventArgs e)
      {
         EnableButtons();
      }

      /// <summary>
      /// Enable/disable buttons.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void lsvManualFills_SelectedIndexChanged(object sender, EventArgs e)
      {
         EnableButtons();
      }

      /// <summary>
      /// Validating text box on lost focus.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A LinkLabelLinkClickedEventArgs that contains the event data.</param>
      private void nudQuantity_LostFocus(object sender, System.EventArgs e)
      {
         nudQuantity.UpButton();
         nudQuantity.DownButton();
      }

      /// <summary>
      /// Turns the "Manual Fills" mode on/off.
      /// </summary>
      /// <param name="sender">he source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void btnManualFill_Click(object sender, EventArgs e)
      {
         m_IsManualFillMode = !m_IsManualFillMode;

         btnManualFill.FlatStyle = m_IsManualFillMode ? FlatStyle.Flat : FlatStyle.Standard;

         tabControlOrders.SelectedIndex = tabControlOrders.TabCount - (m_IsManualFillMode ? 1 : 2);

         chkParked.Enabled = !m_IsManualFillMode;
         grbDuration.Enabled = !m_IsManualFillMode;

         EnableButtons();
      }

      #endregion

      #region Nested Types

      /// <summary>
      /// Used to filter orders by their statuses and show them with different tabs.
      /// </summary>
      [FlagsAttribute]
      private enum eOrderStatusTabs : byte
      {
         Working = 1,
         Filled = 2,
         Canceled = 4,
         Exception = 8,
         Parked = 16,
         All = 32
      }

      /// <summary>
      /// Specifies columns in orders list view control.
      /// The 0 column which is the item(not sub-item), represents the 
      /// original order ID and is processed separately.
      /// </summary>
      private enum eOrdersListColumns : byte
      {
         Side = 1,
         Quantity,
         Instrument,
         Type,
         LimitPrice,
         StopPrice,
         AvgFillPrice,
         Duration,
         GoodTill,
         Status,
         PlaceTime
      }

      /// <summary>
      /// Specifies columns in manual fills list view control.
      /// The 0 column which is the item(not sub-item), represents the 
      /// manual fill ID and is processed separately.
      /// </summary>
      private enum eManualFillsListColumns : byte
      {
         Instrument = 1,
         Side,
         Quantity,
         Price,
         Currency,
         FillDateTime,
         Note,
         StatementDate
      }

      #endregion

   }
}
