﻿using CQG;
using System;
using System.Windows.Forms;

namespace OrderPlacingMetaInfo
{
   public partial class OrderPlacingMetaInfo : Form
   {
      /// <summary>
      /// CQG API instance (CQGCEL).
      /// </summary>
      private CQGCEL m_CEL;

      /// <summary>
      /// CQG account that is used for order placing.
      /// </summary>
      private CQGAccount m_account;

      /// <summary>
      /// CQG instrument that is used for order placing.
      /// </summary>
      private CQGInstrument m_instrument;

      /// <summary>
      /// Form constructor.
      /// </summary>
      public OrderPlacingMetaInfo()
      {
         InitializeComponent();
         InitializeOrderTypes();
         InitializeOrderSides();
         InitializeCQGCEL();
      }

      /// <summary>
      /// Configures CQGCEL.
      /// </summary>
      private void ConfigureCQGCEL()
      {
         m_CEL.APIConfiguration.ReadyStatusCheck = eReadyStatusCheck.rscOff;
         m_CEL.APIConfiguration.UseOrderSide = true;
         m_CEL.APIConfiguration.NewInstrumentMode = true;

         m_CEL.InstrumentResolved += cel_InstrumentResolved;
         m_CEL.OrderChanged += new _ICQGCELEvents_OrderChangedEventHandler(cel_OrderChanged);
         m_CEL.AccountChanged += new _ICQGCELEvents_AccountChangedEventHandler(cel_AccountChanged);
         m_CEL.GWConnectionStatusChanged +=
             new _ICQGCELEvents_GWConnectionStatusChangedEventHandler(cel_GWConnectionStatusChanged);
      }

      /// <summary>
      /// Configures and starts CQGCEL.
      /// </summary>
      private void InitializeCQGCEL()
      {
         m_CEL = new CQGCEL();

         ConfigureCQGCEL();

         m_CEL.Startup();
      }

      /// <summary> 
      /// This event is fired, when some changes occur in the connection 
      /// with the CQG Gateway server.
      /// </summary>
      /// <param name="newStatus"> The current status of the 
      /// connection with the Gateway server.</param>
      private void cel_GWConnectionStatusChanged(eConnectionStatus newStatus)
      {
         if (newStatus == eConnectionStatus.csConnectionUp)
         {
            m_CEL.AccountSubscriptionLevel = eAccountSubscriptionLevel.aslAccountUpdatesAndOrders;
         }
      }

      /// <summary>
      /// Initializes orders' types combo control.
      /// </summary>
      private void InitializeOrderTypes()
      {
         cmbOrderTypes.Items.Add(eOrderType.otMarket);
         cmbOrderTypes.Items.Add(eOrderType.otLimit);
         cmbOrderTypes.Items.Add(eOrderType.otStop);
         cmbOrderTypes.Items.Add(eOrderType.otStopLimit);
         cmbOrderTypes.SelectedIndex = 0;
      }

      /// <summary>
      /// Initializes orders' sides combo control.
      /// </summary>
      private void InitializeOrderSides()
      {
         cmbOrderSides.Items.Add(eOrderSide.osdBuy);
         cmbOrderSides.Items.Add(eOrderSide.osdSell);
         cmbOrderSides.SelectedIndex = 0;
      }

      /// <summary>
      /// This event is fired when the account or position information is changed.
      /// </summary>
      /// <param name="change">Account change type which allows to differentiate the occurred changes.</param>
      /// <param name="account">A CQGAccount object representing the account 
      /// to which the current change refers.</param>
      /// <param name="position">A CQGPosition object representing the position 
      /// to which the current change refers.</param>
      private void cel_AccountChanged(eAccountChangeType change,
                                      CQGAccount account,
                                      CQGPosition position)
      {
         if (account != null)
         {
            lblAccount.Text = account.GWAccountName;
            m_account = account;
         }
      }

      /// <summary>
      /// This event is fired when a new instrument is resolved and subscribed or resolution failed.
      /// </summary>
      /// <param name="symbol">The commodity symbol that was requested by user.</param>
      /// <param name="cqgInstrument">Subscribed CQGInstrument object.</param>
      /// <param name="cqgError">Error happened during a resolution process.</param>
      private void cel_InstrumentResolved(string symbol, CQGInstrument cqgInstrument, CQGError cqgError)
      {
         m_instrument = cqgInstrument;
         txtSymbolName.Text = m_instrument.FullName;

         place();
      }

      /// <summary>
      /// This event is fired when a new order is added to the collection, 
      /// order status is changed or the order is removed from the collection.
      /// </summary>
      /// <param name="change">The type of the occurred change.</param>
      /// <param name="order">CQGOrder object representing the order to which the change refers.</param>
      /// <param name="oldProperties">CQGOrderProperties collection representing the old 
      /// values of the changed order properties.</param>
      /// <param name="fill">CQGFill object representing the last fill of the order.</param>
      /// <param name="objError">CQGError object representing either an error sent by CQG Gateway 
      /// or an intermediate error.</param>
      private void cel_OrderChanged(eChangeType change,
                                    CQGOrder order,
                                    CQGOrderProperties oldProperties,
                                    CQGFill fill,
                                    CQGError objError)
      {
         lblGWOrderID.Text = order.GWOrderID.ToString();
         lblStatus.Text = order.GWStatus.ToString();
      }

      /// <summary>
      /// Subscribes to instrument, based on application GUI state.
      /// </summary>
      private void subscribe()
      {
         // Send subscription request
         CQGInstrumentSubscriptionRequest request = m_CEL.CreateInstrumentRequest();
         request.Symbol = txtSymbolName.Text;

         m_CEL.SubscribeNewInstrument(request);
      }

      /// <summary>
      /// Switches controls.
      /// </summary>
      private void switchControls(bool value)
      {
         btnPlace.Enabled = value;
         txtSymbolName.Enabled = value;
      }

      /// <summary>
      /// Places order, based on application GUI state.
      /// </summary>
      private void place()
      {
         if (m_account == null)
         {
            return;
         }

         eOrderType orderType = (eOrderType)(cmbOrderTypes.SelectedItem);
         eOrderSide orderSide = (eOrderSide)(cmbOrderSides.SelectedItem);

         int quantity = (int)nudQuantity.Value;

         // Default values of prices
         double limitPrice = 0.0;
         double stopPrice = 0.0;

         if (orderType == eOrderType.otLimit || orderType == eOrderType.otStopLimit)
         {
            limitPrice = m_instrument.FromDisplayPrice(txtLimitPrice.Text);
         }

         if (orderType == eOrderType.otStop || orderType == eOrderType.otStopLimit)
         {
            stopPrice = m_instrument.FromDisplayPrice(txtStopPrice.Text);
         }

         CQGOrder order = m_CEL.CreateOrder(orderType, m_instrument, m_account, quantity, orderSide, limitPrice, stopPrice);

         order.Place();

         switchControls(true);
      }

      /// <summary>
      /// Places order.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data.</param>
      private void btnPlace_Click(object sender, EventArgs e)
      {
         switchControls(false);
         subscribe();
      }
   }
}
