﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using CQG;

namespace StrategyOrderPlacing
{
    public partial class frmExecutionPattern : Form
    {
        /// <summary>
        /// Pattern parameter name separator.
        /// </summary>
        private const string NAME_SEPARATOR = " : ";

        /// <summary>
        /// CQGCEL object.
        /// </summary>
        private CQGCEL m_CEL;

        /// <summary>
        /// CQGStrategyDefinition object.
        /// </summary>
        private CQGStrategyDefinition m_StrategyDefinition;

        /// <summary>
        /// CQGExecutionPattern object.
        /// </summary>
        private CQGExecutionPattern m_ExecutionPattern;

        /// <summary>
        /// Execution pattern's current parameter.
        /// </summary>
        private CQGExecutionPatternParameter m_CurrentParam;

        /// <summary>
        /// Hash table to keep mapping of pattern name to pattern.
        /// </summary>
        private Dictionary<string, CQGExecutionPattern> m_Patterns;

        /// <summary>
        /// Execution pattern name.
        /// </summary>
        private string m_PatternName;

        /// <summary>
        /// Flag indicates whether to create new execution pattern or to modify existing one.
        /// </summary>
        private bool m_IsNew;

        /// <summary>
        /// Form constructor.
        /// </summary>
        /// <param name="cel">CQGCEL object.</param>
        /// <param name="strategyDefinition">CQGStrategyDefintion object.</param>
        /// <param name="patterns">Hashtable containg mapping of pattern name to pattern.</param>
        /// <param name="patternName">Execution pattern name.</param>
        public frmExecutionPattern(CQGCEL cel, 
                                   CQGStrategyDefinition strategyDefinition, 
                                   Dictionary<string, CQGExecutionPattern> patterns,
                                   string patternName)
        {
            InitializeComponent();

            m_CEL = cel;
            m_StrategyDefinition = strategyDefinition;
            m_Patterns = patterns;
            m_PatternName = patternName;
            m_IsNew = patternName.Length == 0;
            if (m_Patterns.ContainsKey(patternName))
            {
                m_ExecutionPattern = (CQGExecutionPattern)m_Patterns[patternName];
            }

            InitializeControls();
        }

        /// <summary>
        /// Gets pattern name.
        /// </summary>
        public string PatternName
        {
            get { return m_PatternName; }
        }

        /// <summary>
        /// Gets execution pattern.
        /// </summary>
        public CQGExecutionPattern ExecutionPattern
        {
            get { return m_ExecutionPattern; }
        }

        #region Form controls event handlers

        /// <summary>
        /// Occurs when execution pattern name is changed.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data.</param>
        private void txtName_TextChanged(object sender, EventArgs e)
        {
            EnableControls();
        }

        /// <summary>
        /// Occurs when execution pattern parameter name is changed.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data.</param>
        private void txtParamName_TextChanged(object sender, EventArgs e)
        {
            EnableControls();
        }

        /// <summary>
        /// Occurs when execution pattern parameter add button is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data.</param>
        private void btnAdd_Click(object sender, EventArgs e)
        {
            try
            {
                CQGExecutionPatternParameters patternParams;
                m_CurrentParam = FindParameterPath(tvPatternTree.SelectedNode);
                if (m_CurrentParam == null)
                {
                    patternParams = m_ExecutionPattern.Parameters;
                }
                else
                {
                    patternParams = m_CurrentParam.Parameters;
                }

                // Get parameter name, type and value
                string paramName = txtParamName.Text.Trim();
                eExecutionPatternParameterType paramType = (eExecutionPatternParameterType)cmbParamType.SelectedItem;
                string paramValue = txtParamValue.Text.Trim();
                CQGExecutionPatternParameter patternParam = patternParams[paramName];

                if (patternParam == null)
                {
                    // Add param to the collection
                    patternParam = patternParams.Add(paramName, paramType, paramValue);
                    AddParamToView(patternParam, tvPatternTree.SelectedNode);
                }
                else
                {
                    ErrorHandler.ShowErrorMessage("Parameter with the same name already exists", "Add pattern parameter", this);
                }

                EnableControls();
            }
            catch (Exception ex)
            {
                ErrorHandler.HandleError("frmExecutionPattern", "btnAdd_Click", ex);
            }
        }

        /// <summary>
        /// Occurs when default pattern checkbox is checked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data.</param>
        private void chkDefaultPattern_CheckedChanged(object sender, EventArgs e)
        {
            cmbType.Enabled = chkDefaultPattern.Checked;
        }

        /// <summary>
        /// Occurs when create execution pattern button is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data.</param>
        private void btnCreate_Click(object sender, EventArgs e)
        {
            m_PatternName = txtName.Text;
            if (m_Patterns.ContainsKey(m_PatternName))
            {
                ErrorHandler.ShowErrorMessage("Pattern with the same name already exists", "Create execution pattern", this);
                return;
            }

            try
            {
                // Add root of the tree
                TreeNode root = AddItemToTree(m_PatternName, null);

                if (!chkDefaultPattern.Checked)
                {
                    m_ExecutionPattern = m_CEL.CreateExecutionPattern(null, eOrderType.otUndefined);
                }
                else
                {
                    eOrderType type = (eOrderType)cmbType.SelectedItem;
                    m_ExecutionPattern = m_CEL.CreateExecutionPattern(m_StrategyDefinition, type);

                    ShowPatternParameters(m_ExecutionPattern.Parameters, root);
                }

                tvPatternTree.SelectedNode = tvPatternTree.Nodes[0];
                m_Patterns[m_PatternName] = m_ExecutionPattern;
                EnableControls();
            }
            catch (Exception ex)
            {
                ErrorHandler.HandleError("frmExecutionPattern", "btnCreate_Click", ex);
            }
        }

        /// <summary>
        /// Occurs after execution pattern parameter is selected.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">A TreeViewEventArgs that contains the event data.</param>
        private void tvPatternTree_AfterSelect(object sender, TreeViewEventArgs e)
        {
            EnableControls();
        }

        /// <summary>
        /// Occurs when remove execution pattern parameter button is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data.</param>
        private void btnRemove_Click(object sender, EventArgs e)
        {
            TreeNode selectedNode = tvPatternTree.SelectedNode;
            TreeNode parentNode = selectedNode.Parent;
            string name = selectedNode.Text.Substring(0, selectedNode.Text.IndexOf(NAME_SEPARATOR));
            if (parentNode.Parent == null)
            {
                m_ExecutionPattern.Parameters.Remove(name);
            }
            else
            {
                FindParameterPath(parentNode).Parameters.Remove(name);
            }

            // Removes currently selected node
            tvPatternTree.Nodes.Remove(selectedNode);
        }

        /// <summary>
        /// Occurs when remove execution pattern all parameters button is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data.</param>
        private void btnRemoveAll_Click(object sender, EventArgs e)
        {
            for (int i = m_ExecutionPattern.Parameters.Count - 1; i >= 0 ; --i)
            {
                m_ExecutionPattern.Parameters.Remove(i);
            }
            tvPatternTree.Nodes[0].Nodes.Clear();
            EnableControls();
        }

        /// <summary>
        /// Occurs when execution pattern parameter type is changed from combobox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">An EventArgs that contains the event data.</param>
        private void cmbParamType_SelectedIndexChanged(object sender, EventArgs e)
        {
            txtParamValue.Text = string.Empty;
            EnableControls();
        }

        #endregion

        #region Auxiliary methods

        /// <summary>
        /// Enables controls.
        /// </summary>
        private void EnableControls()
        {
            bool patternNameExists = txtName.Text.Trim().Length > 0;
            bool patternExists = m_ExecutionPattern != null;
            bool hasParam = tvPatternTree.Nodes.Count > 0;
            bool rootSelected = tvPatternTree.SelectedNode != null && tvPatternTree.SelectedNode.Parent != null;
            bool collectionType = IsCollectionType((eExecutionPatternParameterType)cmbParamType.SelectedItem);

            // Initialize pattern related controls
            txtName.Enabled = m_IsNew && !patternExists;
            btnCreate.Enabled = txtName.Enabled && patternNameExists;
            chkDefaultPattern.Enabled = !patternExists && m_StrategyDefinition != null;
            cmbType.Enabled = chkDefaultPattern.Checked && !patternExists;

            bool paramNameExists = txtParamName.Text.Trim().Length > 0;
            // Initialize parameter related controls
            txtParamName.Enabled = patternExists;
            cmbParamType.Enabled = patternExists && paramNameExists;
            txtParamValue.Enabled = cmbParamType.Enabled && !collectionType;
            btnAdd.Enabled = cmbParamType.Enabled;
            btnRemove.Enabled = patternExists && rootSelected && hasParam;
            btnRemoveAll.Enabled = patternExists && hasParam;
        }

        /// <summary>
        /// Initializes order type combobox.
        /// </summary>
        private void InitializeOrderTypeCombobox()
        {
            cmbParamType.Items.Clear();
            cmbType.Items.Add(eOrderType.otMarket);
            cmbType.Items.Add(eOrderType.otLimit);
            cmbType.Items.Add(eOrderType.otStop);

            cmbType.SelectedIndex = 0;
        }

        /// <summary>
        /// Initializes pattern parameter type combobox.
        /// </summary>
        private void InitializePatternParameterTypeCombobox()
        {
            cmbParamType.Items.Clear();
            cmbParamType.Items.Add(eExecutionPatternParameterType.epptBool);
            cmbParamType.Items.Add(eExecutionPatternParameterType.epptInt32);
            cmbParamType.Items.Add(eExecutionPatternParameterType.epptDouble);
            cmbParamType.Items.Add(eExecutionPatternParameterType.epptString);
            cmbParamType.Items.Add(eExecutionPatternParameterType.epptObject);
            cmbParamType.Items.Add(eExecutionPatternParameterType.epptCollection);
            cmbParamType.Items.Add(eExecutionPatternParameterType.epptCollectionItem);

            cmbParamType.SelectedIndex = 0;
        }

        /// <summary>
        /// Initializes all controls.
        /// </summary>
        private void InitializeControls()
        {
            InitializeOrderTypeCombobox();
            InitializePatternParameterTypeCombobox();

            if (!m_IsNew)
            {
                txtName.Text = m_PatternName;
                ShowPatternParameters(m_ExecutionPattern.Parameters, AddItemToTree(m_PatternName, null));
            }

            EnableControls();
        }

        /// <summary>
        /// Adds parameter to the tree view.
        /// </summary>
        /// <param name="patternParam">Pattern parameter to add.</param>
        /// <param name="parent">Parent node.</param>
        private void AddParamToView(CQGExecutionPatternParameter patternParam, TreeNode parent)
        {
            eExecutionPatternParameterType type = patternParam.Type;
            string treeItemName = patternParam.Name.ToString() + NAME_SEPARATOR +
                                  TypeToString(type) + NAME_SEPARATOR +
                                  patternParam.Value.ToString();

            TreeNode node = AddItemToTree(treeItemName, parent);

            if (IsCollectionType(type))
            {
                ShowPatternParameters(patternParam.Parameters, node);
            }
        }

        /// <summary>
        /// Adds item to tree.
        /// </summary>
        /// <param name="itemName">Item name.</param>
        /// <param name="parentNode">Parent node.</param>
        /// <returns></returns>
        private TreeNode AddItemToTree(string itemName, TreeNode parentNode)
        {
            // Adds new node as a child node of the currently selected node.
            TreeNode newNode = new TreeNode(itemName);
            if (parentNode == null)
            {
                tvPatternTree.Nodes.Add(newNode);
            }
            else
            {
                parentNode.Nodes.Add(newNode);
            }

            return newNode;
        }

        /// <summary>
        /// Shows pattern's all parameters.
        /// </summary>
        /// <param name="parameters">Execution pattern parameters.</param>
        /// <param name="parent">Parent node.</param>
        private void ShowPatternParameters(CQGExecutionPatternParameters parameters, TreeNode parent)
        {
            foreach (CQGExecutionPatternParameter param in parameters)
            {
                if (param != null)
                {
                    AddParamToView(param, parent);
                }
            }

            EnableControls();
        }

        /// <summary>
        /// Finds parameter path.
        /// </summary>
        /// <param name="node">Node to find in execution pattern parameters.</param>
        /// <returns>Returns execution pattern parameter wich corresponds to the given node.</returns>
        private CQGExecutionPatternParameter FindParameterPath(TreeNode node)
        {
            TreeNode item = node;
            Stack names = new Stack();

            while (item != null)
            {
                names.Push(item.Text);
                item = item.Parent;
            }

            if (names.Count != 0)
            {
                // removing execution pattern name.
                names.Pop();
                return FindByName(m_ExecutionPattern.Parameters, names);
            }

            return null;
        }

        /// <summary>
        /// Finds execution pattern parameter by its name.
        /// </summary>
        /// <param name="parameters">Execution pattern parameters.</param>
        /// <param name="names">Path from tree root to current parameter name.</param>
        /// <returns>Returns execution pattern parameter.</returns>
        private CQGExecutionPatternParameter FindByName(CQGExecutionPatternParameters parameters, Stack names)
        {
            CQGExecutionPatternParameter parameter = null;
            if (names.Count != 0)
            {
                string name = names.Peek().ToString();
                int index = name.IndexOf(NAME_SEPARATOR);
                name = name.Substring(0, index);
                foreach (CQGExecutionPatternParameter param in parameters)
                {
                    if (param.Name == name)
                    {
                        parameter = param;
                        break;
                    }
                }

                names.Pop();
            }

            if (names.Count == 0)
            {
                // Current parameter is a leaf node (simple parameter) or
                // non-leaf node is selected
                return parameter;
            }

            // Found non simple parameter, getting its parameters collection for future search recursive call
            return FindByName(parameter.Parameters, names);
        }

        /// <summary>
        /// Returns true if execution pattern parameter type is collection type or object.
        /// </summary>
        /// <param name="type">Execution pattern parameter type</param>
        /// <returns>Returns true if execution pattern parameter type is collection type or object.</returns>
        private bool IsCollectionType(eExecutionPatternParameterType type)
        {
            return (type == eExecutionPatternParameterType.epptCollection ||
                    type == eExecutionPatternParameterType.epptCollectionItem ||
                    type == eExecutionPatternParameterType.epptObject);
        }

        /// <summary>
        /// Converts execution pattern parameter type to string.
        /// </summary>
        /// <param name="type">Execution pattern parameter type.</param>
        /// <returns>Execution pattern parameter as string.</returns>
        private string TypeToString(eExecutionPatternParameterType type)
        {
            return type.ToString().Substring(4);
        }

        /// <summary>
        /// Converts string to execution parameter type.
        /// </summary>
        /// <param name="typeStr">Types representation as string.</param>
        /// <returns></returns>
        private eExecutionPatternParameterType StringToType(string typeStr)
        {
            switch (typeStr.ToLower())
            {
                case "bool":
                case "int32":
                case "double":
                case "string":
                case "object":
                case "collection":
                case "collectionitem":
                    return (eExecutionPatternParameterType)Enum.Parse(typeof(eExecutionPatternParameterType), "eppt" + typeStr, true);
                default:
                    throw new ArgumentException("Unknown parameter type.", "typeStr");
            }
        }

        #endregion
    }
}
