﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using CQG;

namespace StrategyOrderPlacing
{
   public partial class frmStrategyOrderPlacing : Form
   {
      /// <summary>
      /// Constant value to identify non applicable entries/values.
      /// </summary>
      internal const string N_A = "N/A";

      /// <summary>
      /// CQG API instance (CQGCEL).
      /// </summary>
      private CQGCEL m_CEL;

      /// <summary>
      /// Current execution pattern.
      /// </summary>
      private CQGExecutionPattern m_CurrentPattern;

      /// <summary>
      /// Hash table to keep mapping of account descriptive name to account GWID.
      /// </summary>
      private readonly Dictionary<string, int> m_AccountsMap = new Dictionary<string, int>(20);

      /// <summary>
      /// Hash table to keep mapping of instrument full name to strategy definition ID.
      /// </summary>
      private readonly Dictionary<string, string> m_DefinitionsMap = new Dictionary<string, string>(20);

      /// <summary>
      ///  Hash table to keep mapping of pattern name to pattern.
      /// </summary>
      private readonly Dictionary<string, CQGExecutionPattern> m_Patterns = new Dictionary<string, CQGExecutionPattern>(10);

      /// <summary>
      /// Hash table to keep mapping of instrument alias to full name.
      /// </summary>
      private readonly Dictionary<string, string> m_AliasToName = new Dictionary<string, string>(20);

      /// <summary>
      /// Form constructor.
      /// </summary>
      public frmStrategyOrderPlacing()
      {
         try
         {
            // Required for Windows Form Designer support
            InitializeComponent();

            InitializeOrdersList(lsvOrders);
            InitializeOrdersList(lsvLegOrders);
            InitializeOrderTypesCombobox();
            InitializeCancellationInstructionCombobox();
            SetOrderPrices(null, eOrderType.otUndefined);

            ClearInstrumentData();

            // Initializes connection controls
            cel_DataConnectionStatusChanged(eConnectionStatus.csConnectionDown);
            cel_GWConnectionStatusChanged(eConnectionStatus.csConnectionDown);

            // Disable buttons
            ChangeDataButtonsStatuses();
            ChangeGWButtonsStatuses();
         }
         catch (Exception ex)
         {
            throw new TypeInitializationException("frmStrategyOrderPlacing", ex);
         }
      }

      #region CQGCEL event handlers

      /// <summary> 
      /// This event is fired, when some changes occur in the connection 
      /// with the CQG data server.
      /// </summary>
      /// <param name="newStatus"> The current status of the 
      /// connection with the Data server.</param>
      private void cel_DataConnectionStatusChanged(eConnectionStatus newStatus)
      {
         try
         {
            lblDataConnection.Text = "Data Connection is ";
            ConnectionStatusChanged(lblDataConnection, newStatus);
            ChangeDataControlsStatuses();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "cel_DataConnectionStatusChanged", ex);
         }
      }

      /// <summary> 
      /// This event is fired, when some changes occur in the connection 
      /// with the CQG Gateway server.
      /// </summary>
      /// <param name="newStatus"> The current status of the 
      /// connection with the Gateway server.</param>
      private void cel_GWConnectionStatusChanged(eConnectionStatus newStatus)
      {
         try
         {
            if (newStatus == eConnectionStatus.csConnectionUp)
            {
               m_CEL.AccountSubscriptionLevel = eAccountSubscriptionLevel.aslAccountUpdatesAndOrders;
            }

            lblGWConnection.Text = "GW Connection is ";
            ConnectionStatusChanged(lblGWConnection, newStatus);

            // Disable controls until accounts are reloaded
            ChangeGWControlsStatuses();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "cel_GWConnectionStatusChanged", ex);
         }
      }

      /// <summary>
      /// This event is fired, when CQGCEL detects some abnormal discrepancy between data expected and data received.
      /// </summary>
      /// <param name="obj">Object, in which the error occurred.</param>
      /// <param name="errorDescription">String, describing the error.</param>
      private void cel_DataError(object obj, string errorDescription)
      {
         CQGError cqgErr = obj as CQGError;
         if (m_CEL.IsValid(cqgErr))
         {
            if (cqgErr.Code == 102)
            {
               errorDescription += "Restart the application.";
            }
            else if (cqgErr.Code == 125)
            {
               errorDescription += "Turn on CQG Client and restart the application.";
            }
         }

         MessageBox.Show(errorDescription, "Data Error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
      }

      /// <summary>
      /// This event is fired when the account or position information is changed.
      /// </summary>
      /// <param name="change">Account change type which allows to differentiate the occurred changes.</param>
      /// <param name="account">A CQGAccount object representing the account 
      /// to which the current change refers.</param>
      /// <param name="position">A CQGPosition object representing the position 
      /// to which the current change refers.</param>
      private void cel_AccountChanged(eAccountChangeType change,
                                      CQGAccount account,
                                      CQGPosition position)
      {
         try
         {
            if (change == eAccountChangeType.actAccountsReloaded)
            {
               FillAccountsList();

               ChangeGWControlsStatuses();
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "cel_AccountChanged", ex);
         }
      }

      /// <summary>
      /// This event is fired when a new instrument is resolved and subscribed.
      /// </summary>
      /// <param name="symbol">The commodity symbol that was requested by user in NewInstrument method.</param>
      /// <param name="instrument">Subscribed CQGInstrument object.</param>
      /// <param name="cqgError">Error happened during a resolution process.</param>
      private void cel_InstrumentResolved(string symbol, CQGInstrument instrument, CQGError cqgError)
      {
         try
         {
            if (m_CEL.IsValid(cqgError))
            {
               ErrorHandler.HandleError("frmStrategyOrderPlacing", "cel_InstrumentResolved", cqgError);
               return;
            }

            if (lstInstruments.Items.Contains(instrument.FullName))
            {
               return;
            }

            int index = lstInstruments.Items.Add(instrument.FullName);
            lstInstruments.SelectedIndex = index;
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "CEL_InstrumentResolved", ex);
         }
      }

      /// <summary>
      /// This event is fired when any of the instrument quotes or dynamic instrument properties are changed.
      /// </summary>
      /// <param name="instrument">Changed instrument.</param>
      /// <param name="quotes">Collection of changed quotes.</param>
      /// <param name="props">Collection of changed dynamic properties. </param>
      private void cel_InstrumentChanged(CQGInstrument instrument,
                                         CQGQuotes quotes,
                                         CQGInstrumentProperties props)
      {
         try
         {
            CQGInstrument selInstrument = GetSelectedInstrument();
            if (selInstrument == null)
            {
               return;
            }

            if (selInstrument.FullName == instrument.FullName)
            {
               DisplayInstrumentProperties(instrument);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "cel_InstrumentChanged", ex);
         }
      }

      /// <summary>
      /// This event is fired if a query has some progress: either new results arrived or its status is changed.
      /// </summary>
      /// <param name="cqg_orders_query">The CQGOrdersQuery object, to which the change refers.
      /// This is the same object that was returned by QueryOrders.</param>
      /// <param name="cqgError">CQGError object representing the last error occurred, while processing this order query. If no error has occurred, 
      /// the parameter value is either Nothing or invalid error object depending on the UsedFromATLClient setting of the configuration.
      /// The parameter can be checked for validity via the CQGCEL.IsValid method.</param>
      /// <remarks>
      /// The results of a query are received via this event. There may be more than one event due to the fact
      /// that results can be sent via more than one chunk of orders.
      /// Results are represented as CQGOrders collection available in Orders property of CQGOrdersQuery object.
      /// If the results were sent via more than one chunk the last arrived chunk is available from LastChunk property and
      /// also all previously received chunks, including the last one, are already added to the Orders collection of current query.
      /// </remarks>
      private void cel_OnQueryProgress(CQGOrdersQuery cqg_orders_query, CQGError cqgError)
      {
         const string IS_STRATEGY = "IsStrategy";
         const string YES = "yes";

         try
         {
            if (cqg_orders_query.Status != eRequestStatus.rsSuccess || cqg_orders_query.Orders.Count < 1)
            {
               return;
            }

            if (m_CEL.IsValid(cqgError))
            {
               MessageBox.Show(cqgError.Description, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
               return;
            }

            foreach (CQGOrder curOrder in cqg_orders_query.Orders)
            {
               if (IsStrategyOrder(curOrder))
               {
                  curOrder.Tag[IS_STRATEGY] = YES;
               }
            }

            CQGOrders strategyOrders = cqg_orders_query.Orders.SelectByTag(IS_STRATEGY, YES);

            if (strategyOrders != null && strategyOrders.Count > 0)
            {
               strategyOrders.AddToLiveOrders();

               if (lstInstruments.Items.Count > 0)
               {
                  DisplayOrders();
               }
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "cel_OnQueryProgress", ex);
         }
      }

      /// <summary>
      /// This event is fired when a new order is added to the collection, 
      /// order status is changed or the order is removed from the collection.
      /// </summary>
      /// <param name="change">The type of the occurred change.</param>
      /// <param name="order">CQGOrder object representing the order to which the change refers.</param>
      /// <param name="oldProperties">CQGOrderProperties collection representing the old 
      /// values of the changed order properties.</param>
      /// <param name="fill">CQGFill object representing the last fill of the order.</param>
      /// <param name="cqgError">CQGError object representing either an error sent by CQG Gateway 
      /// or an intermediate error.</param>
      private void cel_OrderChanged(eChangeType change,
                                    CQGOrder order,
                                    CQGOrderProperties oldProperties,
                                    CQGFill fill,
                                    CQGError cqgError)
      {
         try
         {
            if (!IsObservedOrder(order))
            {
               return;
            }

            if (m_CEL.Orders.get_ItemByGuid(order.GUID) == null)
            {
               return;
            }

            if (m_CEL.IsValid(cqgError))
            {
               MessageBox.Show(cqgError.Description, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
               return;
            }

            // Get current selected account
            CQGAccount account = GetSelectedAccount();
            if (account == null)
            {
               return;
            }

            // Check if the order belongs to selected account
            if (order.Account.GWAccountID == account.GWAccountID)
            {
               DisplayOrder(change, order);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "cel_OrderChanged", ex);
         }
      }

      /// <summary>
      /// This event is fired when strategy definition request is resolved.
      /// </summary>
      /// <param name="cqg_definition">Strategy definition</param>
      /// <param name="cqg_error">Error occured during the request</param>
      private void cel_StrategyDefinitionProgress(CQGStrategyDefinition cqg_definition,
                                                  CQGError cqg_error)
      {
         if (m_CEL.IsValid(cqg_error))
         {
            MessageBox.Show(cqg_error.Description, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
         }

         // Update strategy definition data for selected instrument
         if (lstInstruments.SelectedItem.ToString() == cqg_definition.RequestString)
         {
            DisplayStrategyDefinition(cqg_definition);

            if (cqg_definition.Status == eStrategyRequestStatus.srsSuccess)
            {
               m_AliasToName[cqg_definition.Symbol] = cqg_definition.RequestString;
            }
         }

         ChangeDataButtonsStatuses();
      }

      #endregion

      #region Auxiliary methods

      /// <summary>
      /// Configures and starts CQGCEL.
      /// </summary>
      private void InitializeCQGCEL()
      {
         m_CEL = new CQGCEL();

         // Configure CQG API. Based on this configuration CQG API works differently.
         m_CEL.APIConfiguration.CollectionsThrowException = false;
         m_CEL.APIConfiguration.ReadyStatusCheck = eReadyStatusCheck.rscOff;
         m_CEL.APIConfiguration.TimeZoneCode = eTimeZone.tzCentral;

         // Handle following events
         m_CEL.DataConnectionStatusChanged +=
             new _ICQGCELEvents_DataConnectionStatusChangedEventHandler(cel_DataConnectionStatusChanged);
         m_CEL.GWConnectionStatusChanged +=
             new _ICQGCELEvents_GWConnectionStatusChangedEventHandler(cel_GWConnectionStatusChanged);
         m_CEL.DataError +=
             new _ICQGCELEvents_DataErrorEventHandler(cel_DataError);
         m_CEL.AccountChanged +=
             new _ICQGCELEvents_AccountChangedEventHandler(cel_AccountChanged);
         m_CEL.InstrumentResolved +=
             new _ICQGCELEvents_InstrumentResolvedEventHandler(cel_InstrumentResolved);
         m_CEL.InstrumentChanged +=
             new _ICQGCELEvents_InstrumentChangedEventHandler(cel_InstrumentChanged);
         m_CEL.OnQueryProgress +=
             new _ICQGCELEvents_OnQueryProgressEventHandler(cel_OnQueryProgress);
         m_CEL.OrderChanged +=
             new _ICQGCELEvents_OrderChangedEventHandler(cel_OrderChanged);
         m_CEL.StrategyDefinitionProgress +=
             new _ICQGCELEvents_StrategyDefinitionProgressEventHandler(cel_StrategyDefinitionProgress);

         // Start CQGCEL
         m_CEL.Startup();
      }

      /// <summary>
      /// Initializes orders list view control and tab control used to 
      /// filter orders by their status.
      /// </summary>
      private void InitializeOrdersList(ListView listViewOrders)
      {
         // Add columns to list (this columns represent the sub-items of the list item)
         Type enumType = typeof(eOrdersListColumns);
         Array enumValues = Enum.GetValues(enumType);
         foreach (eOrdersListColumns val in enumValues)
         {
            listViewOrders.Columns.Add(val.ToString(), 70, HorizontalAlignment.Center);
         }
      }

      /// <summary>
      /// Initializes orders' types combobox control.
      /// </summary>
      private void InitializeOrderTypesCombobox()
      {
         cmbOrderTypes.Items.Clear();
         cmbOrderTypes.Items.Add(eOrderType.otMarket);
         cmbOrderTypes.Items.Add(eOrderType.otLimit);
         cmbOrderTypes.Items.Add(eOrderType.otStop);
         cmbOrderTypes.SelectedIndex = 0;
      }

      /// <summary>
      /// Initializes order's cancellation instruction combobox control.
      /// </summary>
      private void InitializeCancellationInstructionCombobox()
      {
         cmbCancelInstr.Items.Clear();
         cmbCancelInstr.Items.Add(eOrderCancelationType.octUndefined);
         cmbCancelInstr.Items.Add(eOrderCancelationType.octCancelAllLegOrders);
         cmbCancelInstr.Items.Add(eOrderCancelationType.octCancelPrimaryLegOrders);
      }

      /// <summary>
      /// Clears all data.
      /// </summary>
      private void ClearInstrumentData()
      {
         // Clear quotes
         lblAskPrice.Text = N_A;
         lblAskVolume.Text = N_A;
         lblBidPrice.Text = N_A;
         lblBidVolume.Text = N_A;
         lblTradePrice.Text = N_A;
         lblTradeVolume.Text = N_A;
      }

      /// <summary>
      /// Clears strategy definition data
      /// </summary>
      private void ClearStrategyDefintionData()
      {
         txtClientID.Text = N_A;
         txtGWAlias.Text = N_A;
         txtStatuss.Text = N_A;
      }

      /// <summary>
      /// Sets specified instrument's current trade price to limit/stop prices controls, based on 
      /// specified order type. Also, sets precision and increment steps of controls.
      /// </summary>
      /// <param name="instrument">CQGInstrument object.</param>
      /// <param name="orderType">Order type.</param>
      private void SetOrderPrices(CQGInstrument instrument, eOrderType orderType)
      {
         txtLimitPrice.Text = N_A;
         txtLimitPrice.Enabled = false;
         vsbLimitPrice.Enabled = false;
         chkIcebergStrategy.Enabled = false;

         txtStopPrice.Text = N_A;
         txtStopPrice.Enabled = false;
         vsbStopPrice.Enabled = false;

         if (orderType == eOrderType.otLimit)
         {
            SetValidPrice(instrument, txtLimitPrice);
            txtLimitPrice.Enabled = true;
            vsbLimitPrice.Enabled = true;
            chkIcebergStrategy.Enabled = true;
         }

         if (orderType == eOrderType.otStop)
         {
            SetValidPrice(instrument, txtStopPrice);
            txtStopPrice.Enabled = true;
            vsbStopPrice.Enabled = true;
            chkIcebergStrategy.Enabled = true;
         }

         nudVisibleQuantity.Enabled = chkIcebergStrategy.Enabled;
      }

      /// <summary>
      /// Returns order property values in string representation.
      /// </summary>
      /// <param name="property">CQGOrderProperty object.</param>
      /// <returns>Property value.</returns>
      private string GetOrderProperty(CQGOrderProperty property)
      {
         if (property != null && property.Value != null)
         {
            return property.Value.ToString();
         }

         return N_A;
      }

      /// <summary>
      /// Returns display format of specified price if it is valid, otherwise N/A.
      /// </summary>
      /// <param name="instrument">CQGInstrument object which is used to get display format.</param>
      /// <param name="price">Raw price.</param>
      /// <returns>String representation of specified price.</returns>
      private string GetDisplayPrice(CQGInstrument instrument, double price)
      {
         if (instrument != null && m_CEL.IsValid(price))
         {
            return instrument.ToDisplayPrice(price);
         }

         return N_A;
      }

      /// <summary>
      /// Returns order property values if it is timestamp.
      /// </summary>
      /// <param name="property">CQGOrderProperty object.</param>
      /// <returns>Property value.</returns>
      private string GetTimestampsValue(CQGOrderProperty property)
      {
         if (property != null)
         {
            return ((DateTime)property.Value).ToString("G");
         }

         return N_A;
      }

      /// <summary>
      /// Changes value of current TextBox to valid price for selected instrument.
      /// </summary>
      /// <param name="instrument">CQGInstrument object which is used to get valid price.</param>
      /// <param name="curTextBox">A TextBox component to put result onto.</param>
      private void SetValidPrice(CQGInstrument instrument, TextBox curTextBox)
      {
         if ((instrument != null) && (curTextBox != null))
         {
            if (instrument.Trade.IsValid)
            {
               curTextBox.Text = instrument.ToDisplayPrice(instrument.Trade.Price);
               return;
            }
            if (instrument.Ask.IsValid)
            {
               curTextBox.Text = instrument.ToDisplayPrice(instrument.Ask.Price);
               return;
            }
            if (instrument.Bid.IsValid)
            {
               curTextBox.Text = instrument.ToDisplayPrice(instrument.Bid.Price);
               return;
            }
         }

         curTextBox.Text = N_A;
      }

      /// <summary>
      /// Fills accounts list control with reloaded accounts.
      /// </summary>
      private void FillAccountsList()
      {
         int recentSelectedIndex = cmbAccounts.SelectedIndex;

         cmbAccounts.Items.Clear();

         // Fill accounts to combo and hash table
         string descriptiveName;
         foreach (CQGAccount acc in m_CEL.Accounts)
         {
            descriptiveName = acc.GWAccountName + " (" + acc.GWAccountID + ")";
            cmbAccounts.Items.Add(descriptiveName);
            m_AccountsMap[descriptiveName] = acc.GWAccountID;

            m_CEL.QueryOrders(acc);
         }

         // Select the recent account or the first one
         if (recentSelectedIndex > 0)
         {
            cmbAccounts.SelectedIndex = recentSelectedIndex;
         }
         else if (cmbAccounts.Items.Count > 0)
         {
            cmbAccounts.SelectedIndex = 0;
         }
      }

      /// <summary>
      /// Enables/disables controls connected with Data server.
      /// </summary>
      private void ChangeDataControlsStatuses()
      {
         try
         {
            bool enable = m_CEL != null && m_CEL.IsStarted &&
                          m_CEL.Environment.DataConnectionStatus == eConnectionStatus.csConnectionUp;

            grbStrategies.Enabled = enable;
            grbExecutionPattern.Enabled = enable;

            if (grbExecutionPattern.Enabled)
            {
               txtPatternString.Text = string.Empty;
               cmbPatternName.Items.Add(string.Empty);
            }
         }
         catch (System.Exception ex)
         {
            MessageBox.Show(ex.Message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
         }
      }

      /// <summary>
      /// Enables/disables controls connected with Gateway server
      /// </summary>
      private void ChangeGWControlsStatuses()
      {
         try
         {
            grbAccounts.Enabled = m_CEL != null && m_CEL.IsStarted &&
                                  m_CEL.Environment.GWConnectionStatus == eConnectionStatus.csConnectionUp;

            bool enable = grbAccounts.Enabled && cmbAccounts.Items.Count > 0 && !string.IsNullOrEmpty(txtGWAlias.Text) && txtGWAlias.Text.Trim() != N_A;

            grbStrategyOrders.Enabled = grbAccounts.Enabled;
            grbLegOrders.Enabled = grbAccounts.Enabled;

            grbOrderProperties.Enabled = enable;
            btnBuy.Enabled = enable;
            btnSell.Enabled = enable;
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
         }
      }

      /// <summary>
      /// Enables/disables Modify, Cancel and Transactions buttons
      /// </summary>
      private void ChangeGWButtonsStatuses()
      {
         bool isOrderSelected = (lsvOrders.SelectedItems.Count == 1);

         btnModify.Enabled = isOrderSelected;
         btnCancel.Enabled = isOrderSelected;
         btnTransactions.Enabled = isOrderSelected;
         cmbCancelInstr.Enabled = isOrderSelected;
         btnCancelAll.Enabled = lsvOrders.Items.Count > 0;
         cmbCancelInstr.SelectedIndex = 0;

         if (isOrderSelected)
         {
            CQGOrder order = lsvOrders.SelectedItems[0].Tag as CQGOrder;
            cmbCancelInstr.Enabled = order.CanBeCanceled;
         }

         btnModifyPattern.Enabled = cmbPatternName.SelectedItem != null &&
                                    cmbPatternName.SelectedItem.ToString().Trim() != string.Empty;
      }

      /// <summary>
      /// Enables/disables Define strategy, Remove instrument buttons.
      /// </summary>
      private void ChangeDataButtonsStatuses()
      {
         bool isInstrumentSelected = grbStrategies.Enabled && lstInstruments.SelectedIndex != -1;

         CQGInstrument instrument = GetSelectedInstrument();
         btnDefineStrategy.Enabled = isInstrumentSelected && m_CEL != null && m_CEL.IsStarted &&
                                     m_CEL.Environment.GWConnectionStatus == eConnectionStatus.csConnectionUp &&
                                     instrument != null && !m_DefinitionsMap.ContainsKey(instrument.FullName);


         btnRemove.Enabled = isInstrumentSelected;
      }

      /// <summary>
      /// Displays currently selected instrument data.
      /// </summary>
      /// <param name="Instrument">CQGInstrument object.</param>
      private void DisplayInstrumentProperties(CQGInstrument instrument)
      {
         // Dump instrument quotes
         DisplayQuote(instrument, eQuoteType.qtAsk, lblAskPrice, lblAskVolume);
         DisplayQuote(instrument, eQuoteType.qtBid, lblBidPrice, lblBidVolume);
         DisplayQuote(instrument, eQuoteType.qtTrade, lblTradePrice, lblTradeVolume);
      }

      /// <summary>
      /// Displays instrument quote if valid. Otherwise N/A.
      /// </summary>
      /// <param name="instr">Instrument which quote should be displayed</param>
      /// <param name="quoteType">Quote to display.</param>
      /// <param name="priceLabel">Quote price label.</param>
      /// <param name="volumeLabel">Quote volume label.</param>
      private void DisplayQuote(CQGInstrument instrument,
                                eQuoteType quoteType,
                                Label priceLabel,
                                Label volumeLabel)
      {
         CQGQuote quote = instrument.Quotes[quoteType];

         if (quote != null && quote.IsValid)
         {
            priceLabel.Text = instrument.ToDisplayPrice(quote.Price);
            volumeLabel.Text = quote.Volume.ToString();
         }
         else
         {
            priceLabel.Text = N_A;
            volumeLabel.Text = N_A;
         }
      }

      /// <summary>
      /// Displays strategy definition data
      /// </summary>
      /// <param name="strategyDefinition">Strategy definition which data should be displayed</param>
      private void DisplayStrategyDefinition(CQGStrategyDefinition strategyDefinition)
      {
         if (strategyDefinition != null)
         {
            string clientID = strategyDefinition.ClientId;
            txtClientID.Text = m_CEL.IsValid(clientID) ? clientID : N_A;

            string symbol = strategyDefinition.Symbol;
            txtGWAlias.Text = m_CEL.IsValid(symbol) ? symbol : N_A;
            txtStatuss.Text = StrategyDefinitionStatusToString(strategyDefinition.Status);
         }
      }

      /// <summary>
      /// Displays currently selected account's orders.
      /// </summary>
      private void DisplayOrders()
      {
         // Clear old data
         lsvOrders.Items.Clear();
         lsvLegOrders.Items.Clear();

         // Get current selected account
         CQGAccount account = GetSelectedAccount();
         if (account != null)
         {
            foreach (CQGOrder order in account.Orders)
            {
               DisplayOrder(eChangeType.ctAdded, order);
            }
         }
      }

      /// <summary>
      /// Checks whether the order info is interesting for sample. The sample displays orders info only for selected spread.
      /// </summary>
      private bool IsObservedOrder(CQGOrder order)
      {
         if (lstInstruments.SelectedItem == null)
         {
            return false;
         }

         string tradingExpression = GetOrderProperty(order.Properties[eOrderProperty.opTradingExpression]);

         if (string.IsNullOrEmpty(tradingExpression) || !tradingExpression.Equals(lstInstruments.SelectedItem.ToString()) || order.Properties[eOrderProperty.opExecutionPattern] == null)
         {
            return false;
         }

         return true;
      }

      /// <summary>
      /// Reflects order change in the list.
      /// </summary>
      /// <param name="change">Order change type.</param>
      /// <param name="order">CQGOrder object.</param>
      private void DisplayOrder(eChangeType change, CQGOrder order)
      {
         if (lstInstruments.SelectedItem == null)
         {
            return;
         }

         // Display only synthetic strategy orders
         if (!IsObservedOrder(order))
         {
            return;
         }

         // Find/add order row in the list
         int index;
         if (change == eChangeType.ctAdded)
         {
            index = lsvOrders.Items.Count;
         }
         else
         {
            ListViewItem foundItem = null;
            // Find item in the list
            foreach (ListViewItem item in lsvOrders.Items)
            {
               if (item.Text == order.OriginalOrderID)
               {
                  foundItem = item;
                  break;
               }
            }

            if (foundItem == null)
            {
               // Order may be absent in the list because of its previous status
               index = lsvOrders.Items.Count;
            }
            else
            {
               index = foundItem.Index;
               lsvOrders.Items.Remove(foundItem);
            }
         }

         // Add order if its status satisfies selected tab
         if (change != eChangeType.ctRemoved)
         {
            ListViewItem item = lsvOrders.Items.Insert(index, order.OriginalOrderID);
            DisplayOrderProperties(item, order);
         }
      }

      /// <summary>
      /// Displays order.
      /// </summary>
      /// <param name="order">CQGOrder to display.</param>
      private void DisplayLegOrder(CQGOrder order)
      {
         ListViewItem item = lsvLegOrders.Items.Insert(lsvLegOrders.Items.Count, order.OriginalOrderID);
         DisplayOrderProperties(item, order);
      }

      /// <summary>
      /// Shows cancellation instruction of the order.
      /// </summary>
      /// <param name="order">CQGOrder object.</param>
      private void ShowCancelationInstruction(CQGOrder order)
      {
         if (order != null)
         {
            int iCancelationTypeIndex = order.IsExternal ? (int)eOrderCancelationType.octUndefined :
                                                           (int)order.CancelationInstruction;
            cmbCancelInstr.SelectedIndex = iCancelationTypeIndex;
         }
      }

      /// <summary>
      /// Displays order properties.
      /// </summary>
      /// <param name="item">List view item.</param>
      /// <param name="order">CQGOrder to display.</param>
      private void DisplayOrderProperties(ListViewItem item, CQGOrder order)
      {
         // Add order as a tag
         item.Tag = order;

         // Add sub items for each column
         for (int i = 0; i < Enum.GetNames(typeof(eOrdersListColumns)).Length; ++i)
         {
            item.SubItems.Add(string.Empty);
         }

         item.SubItems[(int)eOrdersListColumns.Side].Text = (order.Quantity > 0) ? "Buy" : "Sell";
         item.SubItems[(int)eOrdersListColumns.Quantity].Text = Math.Abs(order.QuantityFractional).ToString();
         item.SubItems[(int)eOrdersListColumns.Status].Text = order.GWStatus.ToString();
         item.SubItems[(int)eOrdersListColumns.Instrument].Text =
             item.ListView.Name.Equals(lsvOrders.Name) ? GetOrderProperty(order.Properties[eOrderProperty.opTradingExpression]) : order.InstrumentName;
         item.SubItems[(int)eOrdersListColumns.Type].Text = order.Type.ToString();
         item.SubItems[(int)eOrdersListColumns.State].Text = order.State.ToString();
         item.SubItems[(int)eOrdersListColumns.PlaceTime].Text = order.PlaceTime.ToString("G");
         item.SubItems[(int)eOrdersListColumns.LastFillTime].Text = GetTimestampsValue(order.Properties[eOrderProperty.opLastFillTime]);
         item.SubItems[(int)eOrdersListColumns.CanceledTime].Text = GetTimestampsValue(order.Properties[eOrderProperty.opCanceledTime]);

         // Display prices
         CQGInstrument instrument = GetInsturmentByAlias(order.InstrumentName);
         item.SubItems[(int)eOrdersListColumns.LimitPrice].Text = GetDisplayPrice(instrument, order.LimitPrice);
         item.SubItems[(int)eOrdersListColumns.StopPrice].Text = GetDisplayPrice(instrument, order.StopPrice);

         item.SubItems[(int)eOrdersListColumns.AvgFillPrice].Text =
                 GetDisplayPrice(instrument, (double)order.Properties[eOrderProperty.opAverageFillPrice].Value);
      }

      /// <summary>
      /// Gets active instrument: the instrument, which is currently selected.
      /// </summary>
      /// <returns>Reference to active instrument instance.</returns>
      private CQGInstrument GetSelectedInstrument()
      {
         try
         {
            return m_CEL.Instruments[lstInstruments.SelectedItem];
         }
         catch
         {
            return null;
         }
      }

      /// <summary>
      /// Gets instrument by its alias
      /// </summary>
      /// <returns>Reference to the instrument instance.</returns>
      private CQGInstrument GetInsturmentByAlias(string alias)
      {
         if (!m_AliasToName.ContainsKey(alias))
         {
            return null;
         }

         return m_CEL.Instruments[m_AliasToName[alias]];
      }

      /// <summary>
      /// Gets currently selected strategy definition object.
      /// </summary>
      /// <returns>Reference to selected strategy definition instance.</returns>
      private CQGStrategyDefinition GetSelectedStrategyDefinition()
      {
         CQGInstrument instrument = GetSelectedInstrument();
         if (instrument != null)
         {
            try
            {
               return m_CEL.AllStrategyDefinitions[m_DefinitionsMap[instrument.FullName]];
            }
            catch
            {
               return null;
            }
         }

         return null;
      }

      /// <summary>
      /// Gets currently selected account.
      /// </summary>
      /// <returns>Reference to selected account instance.</returns>
      private CQGAccount GetSelectedAccount()
      {
         try
         {
            return m_CEL.Accounts[m_AccountsMap[cmbAccounts.SelectedItem.ToString()]];
         }
         catch
         {
            return null;
         }
      }

      /// <summary>
      /// Places order, based on application GUI state.
      /// </summary>
      /// <param name="orderSide">Order side.</param>
      private void PlaceOrder(eOrderSide orderSide)
      {
         if (orderSide == eOrderSide.osdUndefined)
         {
            throw new ArgumentException("Order side is not defined.");
         }

         CQGAccount account = GetSelectedAccount();
         if (account == null)
         {
            throw new NullReferenceException("No account is selected.");
         }

         CQGInstrument instrument = GetSelectedInstrument();
         if (instrument == null)
         {
            throw new NullReferenceException("No instrument is selected.");
         }

         eOrderType orderType = (eOrderType)(cmbOrderTypes.SelectedItem);

         var quantity = nudQuantity.Value;
         // Order side can be specified in two ways: 
         //      * if UseOrderSide is set in APIConfig then we need to specify side 
         //        explicitly, and order quantity must be greater than 0.
         //      * if setting below is not set the side is detected by order quantity sign.
         //        Negative quantity specifies sell side.
         // Here we use the second one
         if (orderSide == eOrderSide.osdSell)
         {
            quantity *= -1;
         }

         // Default values of prices
         var limitPrice = double.NaN;
         var stopPrice = double.NaN;

         if (orderType == eOrderType.otLimit || orderType == eOrderType.otStopLimit)
         {
            if (txtLimitPrice.Text == N_A)
            {
               throw new ArgumentException("Limit price is not valid.");
            }
            limitPrice = instrument.FromDisplayPrice(txtLimitPrice.Text);
         }

         if (orderType == eOrderType.otStop || orderType == eOrderType.otStopLimit)
         {
            if (txtStopPrice.Text == N_A)
            {
               throw new ArgumentException("Stop price is not valid.");
            }
            stopPrice = instrument.FromDisplayPrice(txtStopPrice.Text);
         }

         var constructor = m_CEL.CreateStrategyOrderConstructor(orderType, eOrderSide.osdUndefined);
         constructor.Properties[eOrderProperty.opTradingExpression].Value = instrument.FullName;
         constructor.Properties[eOrderProperty.opAccountID].Value = account.GWAccountID;
         constructor.Properties[eOrderProperty.opQuantityFractional].Value = quantity;
         constructor.Properties[eOrderProperty.opUEName].Value = string.Empty;
         if (!double.IsNaN(limitPrice))
         {
            constructor.Properties[eOrderProperty.opLimitPrice].Value = limitPrice;
         }
         if (!double.IsNaN(stopPrice))
         {
            constructor.Properties[eOrderProperty.opStopPrice].Value = stopPrice;
         }

         CQGOrder order = constructor.CreateOrder();
         if (chkIcebergStrategy.Checked)
         {
            // Set order visible quantity
            order.Properties[eOrderProperty.opExecutionInstructions].Value = eOrderExecutionInstruction.oeiIceberg;
            order.Properties[eOrderProperty.opVisibleQuantityFractional].Value = nudVisibleQuantity.Value;
         }

         string patternString = txtPatternString.Text.Trim();
         if (!string.IsNullOrEmpty(patternString))
         {
            order.Properties[eOrderProperty.opExecutionPattern].Value = patternString;
         }

         // Place it
         order.Place();
      }

      /// <summary>
      /// Resolves the symbol and subscribes to the instrument.
      /// </summary>
      private void SubscribeInstrument()
      {
         try
         {
            string instrumentName = txtSpreadName.Text.Trim();
            if (instrumentName == string.Empty)
            {
               return;
            }

            // Do not subscribe to instrument if it is not synthetic strategy
            if (!(instrumentName.Length > 6 && instrumentName.Substring(0, 6).ToLower() == "spread") &&
                !(instrumentName.Length > 2 && instrumentName.Substring(0, 2) == "Q."))
            {
               ErrorHandler.ShowErrorMessage("It is allowed to subscribe only to synthetic strategy instruments\n",
                                             "Instrument subscription", this);
               return;
            }

            // Send subscription request
            m_CEL.NewInstrument(instrumentName);

            // Clear specified commodity
            txtSpreadName.Clear();
         }
         catch (Exception ex)
         {
            ErrorHandler.ShowErrorMessage(ex.Message, "Instrument subscription", this);
         }
      }

      /// <summary>
      /// Displays connection status changes on appropriate label controls.
      /// </summary>
      /// <param name="lblStatus">Label which displays GW/Data connection status.</param>
      /// <param name="newStatus">Connection status to display.</param>
      private void ConnectionStatusChanged(Label lblStatus, eConnectionStatus newStatus)
      {
         if (newStatus == eConnectionStatus.csConnectionUp)
         {
            lblStatus.BackColor = System.Drawing.Color.FromArgb(192, 209, 205);
         }
         else
         {
            lblStatus.BackColor = System.Drawing.Color.FromArgb(255, 114, 0);
         }

         lblStatus.Text = lblStatus.Text + ConnectionStatusToString(newStatus);
      }

      /// <summary>
      /// Returns string representation of the connection status.
      /// </summary>
      /// <param name="status">Connection status.</param>
      /// <returns>String representation of the connection status</returns>
      private string ConnectionStatusToString(eConnectionStatus status)
      {
         string connectionStatus;

         switch (status)
         {
            case eConnectionStatus.csConnectionUp:
               connectionStatus = "Up";
               break;
            case eConnectionStatus.csConnectionDelayed:
               connectionStatus = "Delayed";
               break;
            case eConnectionStatus.csConnectionDown:
               connectionStatus = "Down";
               break;
            case eConnectionStatus.csConnectionTrouble:
               connectionStatus = "Trouble";
               break;
            case eConnectionStatus.csConnectionNotLoggedOn:
               connectionStatus = "Not logged on";
               break;
            default:
               throw new ArgumentException("Unknown connection status.", "newStatus");
         }

         return connectionStatus;
      }

      /// <summary>
      /// Returns string representation of the strategy definition status.
      /// </summary>
      /// <param name="status">Strategy definition status.</param>
      /// <returns>String representation of the strategy definition status</returns>
      private string StrategyDefinitionStatusToString(eStrategyRequestStatus requestStatus)
      {
         string status;
         switch (requestStatus)
         {
            case eStrategyRequestStatus.srsNotActive:
               status = "Not Active";
               break;
            case eStrategyRequestStatus.srsPending:
               status = "Pending";
               break;
            case eStrategyRequestStatus.srsSuccess:
               status = "Success";
               break;
            case eStrategyRequestStatus.srsFailed:
               status = "Failed";
               break;
            default:
               throw new ArgumentException("Unknown strategy definition request status.", "requestStatus");
         }

         return status;
      }

      /// <summary>
      /// Checks whether the specified CQGOrder object is an synthetic strategy - main or strategy leg order.
      /// </summary>
      /// <param name="order">The specified CQGOrder object to check for being synthetic strategy order.</param>
      /// <returns>true, if the specified CQGOrder object is synthetic strategy order; otherwise, false.</returns>
      private bool IsStrategyOrder(CQGOrder order)
      {
         if (order != null)
         {
            bool hasExecutionPattern = order.Properties[eOrderProperty.opExecutionPattern] != null &&
                !string.IsNullOrEmpty(((string)order.Properties[eOrderProperty.opExecutionPattern].Value));

            bool hasStrategyOrderGUID = order.Properties[eOrderProperty.opStrategyOrderGUID] != null &&
                !string.IsNullOrEmpty(((string)order.Properties[eOrderProperty.opStrategyOrderGUID].Value));

            return hasExecutionPattern || hasStrategyOrderGUID;
         }

         return false;
      }

      #endregion

      #region Static methods

      /// <summary>
      /// The main entry point for the application.
      /// </summary>
      [STAThread]
      static void Main()
      {
         try
         {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new frmStrategyOrderPlacing());
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "Main", ex);
         }
      }

      /// <summary>
      /// Increments/decrements price in specified control by specified instrument tick size 
      /// when scroll bar is clicked.
      /// </summary>
      /// <param name="instrument">CQGInstrument object.</param>
      /// <param name="priceControl">TextBox control which contains current price.</param>
      /// <param name="e">Scroll bar change event argument.</param>
      internal static void UpDownPrice(CQGInstrument instrument,
                                       TextBox priceControl,
                                       ScrollEventArgs e)
      {
         if (instrument == null || priceControl.Text == N_A)
         {
            return;
         }

         int upDown = 0;
         switch (e.Type)
         {
            case ScrollEventType.SmallIncrement:
            case ScrollEventType.LargeIncrement:
               upDown = -1;
               break;
            case ScrollEventType.SmallDecrement:
            case ScrollEventType.LargeDecrement:
               upDown = 1;
               break;
         }

         // Increase/decrease price by tick
         if (upDown != 0)
         {
            double rawPrice = instrument.FromDisplayPrice(priceControl.Text);
            rawPrice += upDown * instrument.TickSize;
            priceControl.Text = instrument.ToDisplayPrice(rawPrice);
         }
      }

      #endregion

      #region Form controls event handlers

      /// <summary>
      /// Initializes CQGCEL object when form is loaded.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void frmStrategyOrderPlacing_Load(object sender, EventArgs e)
      {
         try
         {
            InitializeCQGCEL();
         }
         catch (System.Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "frmStrategyOrderPlacing_Load", ex);
         }
      }

      /// <summary>
      /// Shutdowns CQGCEL before closing the form.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A FormClosingEventArgs that contains the event data. </param>
      private void frmStrategyOrderPlacing_FormClosing(object sender, FormClosingEventArgs e)
      {
         try
         {
            m_CEL.Shutdown();
         }
         catch
         { }
      }

      /// <summary>
      /// Open CQG API web page.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A LinkLabelLinkClickedEventArgs that contains the event data.</param>
      private void llWeb_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
      {
         System.Diagnostics.Process.Start("http://www.cqg.com/Products/CQG-API.aspx");
      }

      /// <summary>
      /// Shows newly selected account's orders.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void cmbAccounts_SelectedIndexChanged(object sender, EventArgs e)
      {
         try
         {
            DisplayOrders();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "cmbAccounts_SelectedIndexChanged", ex);
         }
      }

      /// <summary>
      /// Enables the Subscribe button when the textbox content is not empty.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void txtSpreadName_TextChanged(object sender, EventArgs e)
      {
         btnSubscribe.Enabled = !string.IsNullOrEmpty(txtSpreadName.Text);
      }

      /// <summary>
      /// Subscribe to the new instrument when the button is clicked.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void btnSubscribe_Click(object sender, EventArgs e)
      {
         SubscribeInstrument();
      }

      /// <summary>
      /// Subscribe to the new instrument when the "Enter" button is clicked.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A KeyEventArgs that contains the event data. </param>
      private void txtSpreadName_KeyDown(object sender, KeyEventArgs e)
      {
         if (e.KeyCode == Keys.Enter)
         {
            SubscribeInstrument();
         }
      }

      /// <summary>
      /// Opens QFormula definition form when the button is clicked.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void btnQFormula_Click(object sender, EventArgs e)
      {
         try
         {
            frmQFormula QFormulaForm = new frmQFormula(m_CEL);
            if (QFormulaForm.ShowDialog(this) == DialogResult.OK)
            {
               txtSpreadName.Text = QFormulaForm.QFormulaDefinitionName;
               cmbPatternName.SelectedIndex = 0;
               txtPatternString.Text = QFormulaForm.QFormulaDefinitionExecutionPattern;
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "btnModify_Click", ex);
         }
      }

      /// <summary>
      /// Used to display newly selected instrument data.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void lstInstruments_SelectedIndexChanged(object sender, EventArgs e)
      {
         if (lstInstruments.Items.Count < 1)
         {
            lsvOrders.Items.Clear();
            lsvLegOrders.Items.Clear();

            return;
         }

         try
         {
            CQGInstrument instrument = GetSelectedInstrument();

            if (instrument == null)
            {
               ClearInstrumentData();
               ClearStrategyDefintionData();
            }
            else
            {
               DisplayInstrumentProperties(instrument);

               if (m_DefinitionsMap.ContainsKey(instrument.FullName))
               {
                  DisplayStrategyDefinition(m_CEL.AllStrategyDefinitions[m_DefinitionsMap[instrument.FullName]]);
               }
               else
               {
                  ClearStrategyDefintionData();
               }

               DisplayOrders();
            }

            // Set new prices for selected instrument
            SetOrderPrices(instrument, (eOrderType)cmbOrderTypes.SelectedItem);
            ChangeDataButtonsStatuses();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "lstInstruments_SelectedIndexChanged", ex);
         }
      }

      /// <summary>
      /// Change the GW controls statuses after GWAlias text change.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void txtGWAlias_TextChanged(object sender, EventArgs e)
      {
         ChangeGWControlsStatuses();
      }

      /// <summary>
      /// Defines strategy for the selected instrument from the list.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void btnDefineStrategy_Click(object sender, EventArgs e)
      {
         try
         {
            CQGInstrument instrument = GetSelectedInstrument();
            if (instrument != null)
            {
               string instrFullName = instrument.FullName;
               CQGStrategyDefinition strategyDefinition = m_CEL.DefineStrategyEx(instrFullName, GetSelectedAccount());
               m_DefinitionsMap[instrFullName] = strategyDefinition.Id;

               DisplayStrategyDefinition(strategyDefinition);
            }
         }
         catch (System.Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "btnDefineStrategy_Click", ex);
         }
      }

      /// <summary>
      /// Removes selected instrument from the list and CQGCEL.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void btnRemove_Click(object sender, EventArgs e)
      {
         try
         {
            CQGInstrument selInstrument = GetSelectedInstrument();
            if (selInstrument == null)
            {
               return;
            }

            // Remove from list
            string instrumentName = selInstrument.FullName;
            lstInstruments.Items.Remove(instrumentName);
            m_DefinitionsMap.Remove(instrumentName);
            if (lstInstruments.Items.Count > 0)
            {
               // Select first in the list
               lstInstruments.SelectedIndex = 0;
            }
            else
            {
               // Nothing to select, clear old data
               ClearInstrumentData();
            }

            // Remove instrument from CEL
            m_CEL.RemoveInstrument(selInstrument);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "btnRemoveInstrument_Click", ex);
         }
      }

      /// <summary>
      /// Used to display newly selected strategy order leg orders.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void lsvOrders_SelectedIndexChanged(object sender, EventArgs e)
      {
         try
         {
            // Clear old data
            lsvLegOrders.Items.Clear();

            // Retrieve currently selected order
            if (lsvOrders.SelectedItems.Count == 1)
            {
               CQGOrder order = lsvOrders.SelectedItems[0].Tag as CQGOrder;
               if (order != null)
               {
                  CQGOrders legOrders = m_CEL.Orders.SelectByProperty(eOrderProperty.opStrategyOrderGUID, order.GUID);
                  foreach (CQGOrder legOrder in legOrders)
                  {
                     DisplayLegOrder(legOrder);
                  }

                  ShowCancelationInstruction(order);
               }
            }

            ChangeGWButtonsStatuses();
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "lsvOrders_SelectedIndexChanged", ex);
         }
      }

      /// <summary>
      /// Opens windows to specify properties and modify order.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void btnModify_Click(object sender, EventArgs e)
      {
         try
         {
            if (lsvOrders.SelectedItems.Count == 1)
            {
               CQGOrder order = lsvOrders.SelectedItems[0].Tag as CQGOrder;
               if (order.CanBeModified)
               {
                  frmModify modForm = new frmModify(m_CEL, order, GetInsturmentByAlias(order.InstrumentName));
                  modForm.ShowDialog(this);
               }
               else
               {
                  MessageBox.Show(this,
                                  "This order can't be modified.",
                                  "Modification message",
                                  MessageBoxButtons.OK,
                                  MessageBoxIcon.Exclamation);
               }
            }
            else
            {
               MessageBox.Show(this,
                               "Please select one order to modify.",
                               "Modification message",
                               MessageBoxButtons.OK,
                               MessageBoxIcon.Exclamation);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "btnModify_Click", ex);
         }
      }

      /// <summary>
      /// Cancels selected orders.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void btnCancel_Click(object sender, EventArgs e)
      {
         try
         {
            if (lsvOrders.SelectedItems.Count == 1)
            {
               CQGOrder order = lsvOrders.SelectedItems[0].Tag as CQGOrder;
               if (order.CanBeCanceled)
               {
                  order.Cancel();
               }
               else
               {
                  MessageBox.Show(this,
                                  "This order can't be canceled.",
                                  "Cancellation message",
                                  MessageBoxButtons.OK,
                                  MessageBoxIcon.Exclamation);
               }
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "btnCancel_Click", ex);
         }
      }

      /// <summary>
      /// Cancels selected account's all orders.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void btnCancelAll_Click(object sender, EventArgs e)
      {
         try
         {
            CQGAccount account = GetSelectedAccount();
            if (account == null)
            {
               return;
            }

            if (account.Orders.Count == 0)
            {
               return;
            }

            DialogResult dlgRes = MessageBox.Show(this,
                                                  "Are you sure you want to cancel all orders?",
                                                  "Account orders cancellation confirmation",
                                                  MessageBoxButtons.YesNo,
                                                  MessageBoxIcon.Question);

            if (dlgRes == DialogResult.Yes)
            {
               m_CEL.CancelAllOrders(account, null, false, false, eOrderSide.osdUndefined);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "btnCancelAll_Click", ex);
         }
      }

      /// <summary>
      /// Sets cancellation instruction of the order
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void cmbCancelInstr_SelectedIndexChanged(object sender, EventArgs e)
      {
         if (lsvOrders.SelectedItems.Count != 0)
         {
            CQGOrder order = lsvOrders.SelectedItems[0].Tag as CQGOrder;
            order.CancelationInstruction = (eOrderCancelationType)cmbCancelInstr.SelectedItem;
         }
      }

      /// <summary>
      /// Shows order transactions form
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void btnTransactions_Click(object sender, EventArgs e)
      {
         try
         {
            if (lsvOrders.SelectedItems.Count == 1)
            {
               CQGOrder order = lsvOrders.SelectedItems[0].Tag as CQGOrder;
               frmTransactions transactionsForm = new frmTransactions(order);
               transactionsForm.ShowDialog(this);
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "btnTransactions_Click", ex);
         }
      }

      /// <summary>
      /// Recalculate prices when order type is changed.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void cmbOrderTypes_SelectedIndexChanged(object sender, EventArgs e)
      {
         try
         {
            SetOrderPrices(GetSelectedInstrument(), (eOrderType)cmbOrderTypes.SelectedItem);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "cmbOrderTypes_SelectedIndexChanged", ex);
         }
      }

      /// <summary>
      /// Enables visible quantity control.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void chkIcebergStrategy_CheckedChanged(object sender, EventArgs e)
      {
         try
         {
            nudVisibleQuantity.Enabled = chkIcebergStrategy.Checked;
         }
         catch (System.Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "chkIcebergStrategy_CheckedChanged", ex);
         }
      }

      /// <summary>
      /// Increments/Decrements limit price.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A ScrollEventArgs that contains the event data. </param>
      private void vsbLimitPrice_Scroll(object sender, ScrollEventArgs e)
      {
         try
         {
            UpDownPrice(GetSelectedInstrument(), txtLimitPrice, e);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "vsbLimitPrice_Scroll", ex);
         }
      }

      /// <summary>
      /// Increments/Decrements stop price.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">A ScrollEventArgs that contains the event data. </param>
      private void vsbStopPrice_Scroll(object sender, ScrollEventArgs e)
      {
         try
         {
            UpDownPrice(GetSelectedInstrument(), txtStopPrice, e);
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "vsbStopPrice_Scroll", ex);
         }
      }

      /// <summary>
      /// Opens execution pattern form.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void btnAddPattern_Click(object sender, EventArgs e)
      {
         try
         {
            frmExecutionPattern executionPatternForm =
                new frmExecutionPattern(m_CEL, GetSelectedStrategyDefinition(), m_Patterns, string.Empty);
            if (executionPatternForm.ShowDialog(this) == DialogResult.OK)
            {
               string patternName = executionPatternForm.PatternName;
               if (patternName != string.Empty)
               {
                  m_CurrentPattern = m_Patterns[patternName];
                  txtPatternString.Text = m_CurrentPattern.PatternString;
                  cmbPatternName.SelectedIndex = cmbPatternName.Items.Add(patternName);
               }
            }
         }
         catch (Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "btnAddPattern_Click", ex);
         }
      }

      /// <summary>
      /// Opens execution pattern form.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void btnModifyPattern_Click(object sender, EventArgs e)
      {

         frmExecutionPattern executionPatternForm =
             new frmExecutionPattern(m_CEL, GetSelectedStrategyDefinition(), m_Patterns, cmbPatternName.SelectedItem.ToString());
         if (executionPatternForm.ShowDialog(this) == DialogResult.OK)
         {
            string patternName = executionPatternForm.PatternName;
            CQGExecutionPattern executionPattern = m_Patterns[patternName];
            txtPatternString.Text = executionPattern.PatternString;
            cmbPatternName.SelectedIndex = cmbPatternName.FindString(patternName);
         }
      }

      /// <summary>
      /// Changes selected execution pattern.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data.</param>
      private void cmbPatternName_SelectedIndexChanged(object sender, EventArgs e)
      {
         try
         {
            string patternName = cmbPatternName.SelectedItem.ToString();
            btnModifyPattern.Enabled = patternName != string.Empty;
            txtPatternString.Enabled = patternName == string.Empty;

            if (m_Patterns.ContainsKey(patternName))
            {
               CQGExecutionPattern executionPattern = m_Patterns[patternName];
               if (executionPattern != null)
               {
                  txtPatternString.Text = executionPattern.PatternString;
               }
            }
         }
         catch (System.Exception ex)
         {
            ErrorHandler.HandleError("frmStrategyOrderPlacing", "cmbPatternName_SelectedIndexChanged", ex);
         }
      }

      /// <summary>
      /// Buys order.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void btnBuy_Click(object sender, EventArgs e)
      {
         try
         {
            PlaceOrder(eOrderSide.osdBuy);
         }
         catch (System.Exception ex)
         {
            ErrorHandler.ShowErrorMessage(ex.Message, "Strategy order placing", this);
         }
      }

      /// <summary>
      /// Sells order.
      /// </summary>
      /// <param name="sender">The source of the event.</param>
      /// <param name="e">An EventArgs that contains the event data. </param>
      private void btnSell_Click(object sender, EventArgs e)
      {
         try
         {
            PlaceOrder(eOrderSide.osdSell);
         }
         catch (System.Exception ex)
         {
            ErrorHandler.ShowErrorMessage(ex.Message, "Strategy order placing", this);
         }
      }

      #endregion

      #region Nested types

      /// <summary>
      /// Specifies columns in orders list view control.
      /// The 0 column which is the item(not sub-item), represents the 
      /// original order ID and is processed separately.
      /// </summary>
      private enum eOrdersListColumns
      {
         OrderID = 0,
         Side,
         Quantity,
         Status,
         Instrument,
         Type,
         LimitPrice,
         StopPrice,
         AvgFillPrice,
         State,
         PlaceTime,
         LastFillTime,
         CanceledTime
      }

      #endregion
   }
}
